@@ -11,12 +11,17 @@
 */
 #include <iostream>
 #include <vector>
+#include <fstream>
 
 #include "zypp/base/LogTools.h"
 #include "zypp/base/DefaultIntegral.h"
 #include "zypp/parser/xml/XmlEscape.h"
 
+#include "zypp/ManagedFile.h"
+#include "zypp/PublicKey.h"
+#include "zypp/MediaSetAccess.h"
 #include "zypp/RepoInfo.h"
+#include "zypp/Glob.h"
 #include "zypp/TriBool.h"
 #include "zypp/Pathname.h"
 #include "zypp/ZConfig.h"
@@ -28,6 +33,13 @@
 #include "zypp/base/InputStream.h"
 #include "zypp/parser/xml/Reader.h"
 
+
+#include "zypp/base/StrMatcher.h"
+#include "zypp/KeyRing.h"
+#include "zypp/TmpPath.h"
+#include "zypp/ZYppFactory.h"
+#include "zypp/ZYppCallbacks.h"
+
 using std::endl;
 using zypp::xml::escape;
 
@@ -43,9 +55,9 @@ namespace zypp
   struct RepoInfo::Impl
   {
     Impl()
-      : _gpgCheck( indeterminate )
-      , _repoGpgCheck( indeterminate )
-      , _pkgGpgCheck( indeterminate )
+      : _rawGpgCheck( indeterminate )
+      , _rawRepoGpgCheck( indeterminate )
+      , _rawPkgGpgCheck( indeterminate )
       , _validRepoSignature( indeterminate )
       ,	keeppackages(indeterminate)
       , _mirrorListForceMetalink(false)
@@ -58,6 +70,7 @@ namespace zypp
 
   public:
     static const unsigned defaultPriority = 99;
+    static const unsigned noPriority = unsigned(-1);
 
     void setProbedType( const repo::RepoType & t ) const
     {
@@ -70,8 +83,31 @@ namespace zypp
     }
 
   public:
-    Pathname licenseTgz() const
-    { return metadatapath.empty() ? Pathname() : metadatapath / path / "license.tar.gz"; }
+    /** Path to a license tarball in case it exists in the repo. */
+    Pathname licenseTgz( const std::string & name_r ) const
+    {
+      Pathname ret;
+      if ( !metadataPath().empty() )
+      {
+	std::string licenseStem( "license" );
+	if ( !name_r.empty() )
+	{
+	  licenseStem += "-";
+	  licenseStem += name_r;
+	}
+
+	filesystem::Glob g;
+	// TODO: REPOMD: this assumes we know the name of the tarball. In fact
+	// we'd need to get the file from repomd.xml (<data type="license[-name_r]">)
+	g.add( metadataPath() / path / ("repodata/*"+licenseStem+".tar.gz") );
+	if ( g.empty() )
+	  g.add( metadataPath() / path / (licenseStem+".tar.gz") );
+
+	if ( !g.empty() )
+	  ret = *g.begin();
+      }
+      return ret;
+    }
 
     const RepoVariablesReplacedUrlList & baseUrls() const
     {
@@ -79,8 +115,8 @@ namespace zypp
       if ( _baseUrls.empty() && ! mlurl.asString().empty() )
       {
         emptybaseurls = true;
-        DBG << "MetadataPath: " << metadatapath << endl;
-	repo::RepoMirrorList rmurls( mlurl, metadatapath, _mirrorListForceMetalink );
+        DBG << "MetadataPath: " << metadataPath() << endl;
+	repo::RepoMirrorList rmurls( mlurl, metadataPath(), _mirrorListForceMetalink );
 	_baseUrls.raw().insert( _baseUrls.raw().end(), rmurls.getUrls().begin(), rmurls.getUrls().end() );
       }
       return _baseUrls;
@@ -93,30 +129,40 @@ namespace zypp
     { return !emptybaseurls && !_baseUrls.empty(); }
 
 
-    void addContent( const std::string & keyword_r )
-    { _keywords.insert( keyword_r ); }
+    const RepoVariablesReplacedUrlList & gpgKeyUrls() const
+    { return _gpgKeyUrls; }
 
-    bool hasContent( const std::string & keyword_r ) const
+    RepoVariablesReplacedUrlList & gpgKeyUrls()
+    { return _gpgKeyUrls; }
+
+
+    const std::set<std::string> & contentKeywords() const
+    { hasContent()/*init if not yet done*/; return _keywords.second; }
+
+    void addContent( const std::string & keyword_r )
+    { _keywords.second.insert( keyword_r ); if ( ! hasContent() ) _keywords.first = true; }
+
+    bool hasContent() const
     {
-      if ( _keywords.empty() && ! metadatapath.empty() )
+      if ( !_keywords.first && ! metadataPath().empty() )
       {
 	// HACK directly check master index file until RepoManager offers
-	// some content probing ans zypepr uses it.
+	// some content probing and zypper uses it.
 	/////////////////////////////////////////////////////////////////
-	MIL << "Empty keywords...." << metadatapath << endl;
+	MIL << "Empty keywords...." << metadataPath() << endl;
 	Pathname master;
-	if ( PathInfo( (master=metadatapath/"/repodata/repomd.xml") ).isFile() )
+	if ( PathInfo( (master=metadataPath()/"/repodata/repomd.xml") ).isFile() )
 	{
 	  //MIL << "GO repomd.." << endl;
 	  xml::Reader reader( master );
 	  while ( reader.seekToNode( 2, "content" ) )
 	  {
-	    _keywords.insert( reader.nodeText().asString() );
+	    _keywords.second.insert( reader.nodeText().asString() );
 	    reader.seekToEndNode( 2, "content" );
 	  }
-	  _keywords.insert( "" );	// valid content in _keywords even if empty
+	  _keywords.first = true;	// valid content in _keywords even if empty
 	}
-	else if ( PathInfo( (master=metadatapath/"/content") ).isFile() )
+	else if ( PathInfo( (master=metadataPath()/"/content") ).isFile() )
 	{
 	  //MIL << "GO content.." << endl;
 	  iostr::forEachLine( InputStream( master ),
@@ -128,31 +174,35 @@ namespace zypp
 				if ( str::split( line_r, std::back_inserter(words) ) > 1
 				  && words[0].length() == 12 /*"REPOKEYWORDS"*/ )
 				{
-				  this->_keywords.insert( ++words.begin(), words.end() );
+				  this->_keywords.second.insert( ++words.begin(), words.end() );
 				}
 				return true; // mult. occurrances are ok.
 			      }
 			      return( ! str::startsWith( line_r, "META " ) );	// no need to parse into META section.
 			    } );
-	  _keywords.insert( "" );
+	  _keywords.first = true;	// valid content in _keywords even if empty
 	}
 	/////////////////////////////////////////////////////////////////
       }
-      return( _keywords.find( keyword_r ) != _keywords.end() );
+      return _keywords.first;
     }
 
-    /** Signature check result needs to be stored/retrieved from _metadatapath.
+    bool hasContent( const std::string & keyword_r ) const
+    { return( hasContent() && _keywords.second.find( keyword_r ) != _keywords.second.end() ); }
+
+    /** Signature check result needs to be stored/retrieved from _metadataPath.
      * Don't call them from outside validRepoSignature/setValidRepoSignature
      */
     //@{
     TriBool internalValidRepoSignature() const
     {
-      if ( ! indeterminate(_validRepoSignature) )		return _validRepoSignature;
+      if ( ! indeterminate(_validRepoSignature) )
+	return _validRepoSignature;
       // check metadata:
-      if ( ! metadatapath.empty() )
+      if ( ! metadataPath().empty() )
       {
-	//TODO: a missing ".repo_gpgcheck" might be plaindir(no Downloader) or not yet refreshed signed repo!
-	TriBool linkval = triBoolFromPath( metadatapath / ".repo_gpgcheck" );
+	// A missing ".repo_gpgcheck" might be plaindir(no Downloader) or not yet refreshed signed repo!
+	TriBool linkval = triBoolFromPath( metadataPath() / ".repo_gpgcheck" );
 	return linkval;
       }
       return indeterminate;
@@ -160,9 +210,9 @@ namespace zypp
 
     void internalSetValidRepoSignature( TriBool value_r )
     {
-      if ( PathInfo(metadatapath).isDir() )
+      if ( PathInfo(metadataPath()).isDir() )
       {
-	Pathname gpgcheckFile( metadatapath / ".repo_gpgcheck" );
+	Pathname gpgcheckFile( metadataPath() / ".repo_gpgcheck" );
 	if ( PathInfo(gpgcheckFile).isExist() )
 	{
 	  TriBool linkval( indeterminate );
@@ -176,12 +226,30 @@ namespace zypp
       _validRepoSignature = value_r;
     }
 
+    /** We definitely have a symlink pointing to "indeterminate" (for repoGpgCheckIsMandatory)?
+     * I.e. user accepted the unsigned repo in Downloader. A test whether `internalValidRepoSignature`
+     * is indeterminate would include not yet checked repos, which is unwanted here.
+     */
+    bool internalUnsignedConfirmed() const
+    {
+      TriBool linkval( true );	// want to see it being switched to indeterminate
+      return triBoolFromPath( metadataPath() / ".repo_gpgcheck", linkval ) && indeterminate(linkval);
+    }
+
     bool triBoolFromPath( const Pathname & path_r, TriBool & ret_r ) const
     {
       static const Pathname truePath( "true" );
       static const Pathname falsePath( "false" );
       static const Pathname indeterminatePath( "indeterminate" );
-      Pathname linkval( filesystem::readlink( path_r ) );
+
+      // Quiet readlink;
+      static const ssize_t bufsiz = 63;
+      static char buf[bufsiz+1];
+      ssize_t ret = ::readlink( path_r.c_str(), buf, bufsiz );
+      buf[ret == -1 ? 0 : ret] = '\0';
+
+      Pathname linkval( buf );
+
       bool known = true;
       if ( linkval == truePath )
 	ret_r = true;
@@ -199,30 +267,72 @@ namespace zypp
 
     //@}
 
+  private:
+    TriBool _rawGpgCheck;	///< default gpgcheck behavior: Y/N/ZConf
+    TriBool _rawRepoGpgCheck;	///< need to check repo sign.: Y/N/(ZConf(Y/N/gpgCheck))
+    TriBool _rawPkgGpgCheck;	///< need to check pkg sign.: Y/N/(ZConf(Y/N/gpgCheck))
+
   public:
-    TriBool _gpgCheck;		///< default gpgcheck behavior: Y/N/ZConf
-    TriBool _repoGpgCheck;	///< need to check repo sign.: Y/N/(ZConf(Y/N/gpgCheck))
-    TriBool _pkgGpgCheck;	///< need to check pkg sign.: Y/N/(ZConf(Y/N/gpgCheck && no valid repo sign.))
+    TriBool rawGpgCheck() const			{ return _rawGpgCheck; }
+    TriBool rawRepoGpgCheck() const		{ return _rawRepoGpgCheck; }
+    TriBool rawPkgGpgCheck() const		{ return _rawPkgGpgCheck; }
+
+    void rawGpgCheck( TriBool val_r )		{ _rawGpgCheck = val_r; }
+    void rawRepoGpgCheck( TriBool val_r )	{ _rawRepoGpgCheck = val_r; }
+    void rawPkgGpgCheck( TriBool val_r )	{ _rawPkgGpgCheck = val_r; }
+
+    bool cfgGpgCheck() const
+    { return indeterminate(_rawGpgCheck) ? ZConfig::instance().gpgCheck() : (bool)_rawGpgCheck; }
+    TriBool cfgRepoGpgCheck() const
+    { return indeterminate(_rawGpgCheck) && indeterminate(_rawRepoGpgCheck) ? ZConfig::instance().repoGpgCheck() : _rawRepoGpgCheck; }
+    TriBool cfgPkgGpgCheck() const
+    { return indeterminate(_rawGpgCheck) && indeterminate(_rawPkgGpgCheck) ? ZConfig::instance().pkgGpgCheck() : _rawPkgGpgCheck; }
+
   private:
     TriBool _validRepoSignature;///< have  signed and valid repo metadata
   public:
     TriBool keeppackages;
-    RepoVariablesReplacedUrl _gpgKeyUrl;
     RepoVariablesReplacedUrl _mirrorListUrl;
     bool                     _mirrorListForceMetalink;
     repo::RepoType type;
     Pathname path;
     std::string service;
     std::string targetDistro;
-    Pathname metadatapath;
-    Pathname packagespath;
+
+    void metadataPath( Pathname new_r )
+    { _metadataPath = std::move( new_r ); }
+
+    void packagesPath( Pathname new_r )
+    { _packagesPath = std::move( new_r ); }
+
+    bool usesAutoMethadataPaths() const
+    { return str::hasSuffix( _metadataPath.asString(), "/%AUTO%" ); }
+
+    Pathname metadataPath() const
+    {
+      if ( usesAutoMethadataPaths() )
+	return _metadataPath.dirname() / "%RAW%";
+      return _metadataPath;
+    }
+
+    Pathname packagesPath() const
+    {
+      if ( _packagesPath.empty() && usesAutoMethadataPaths() )
+	return _metadataPath.dirname() / "%PKG%";
+      return _packagesPath;
+    }
+
     DefaultIntegral<unsigned,defaultPriority> priority;
     mutable bool emptybaseurls;
-    repo::RepoVariablesUrlReplacer replacer;
 
   private:
+    Pathname _metadataPath;
+    Pathname _packagesPath;
+
     mutable RepoVariablesReplacedUrlList _baseUrls;
-    mutable std::set<std::string> _keywords;
+    mutable std::pair<FalseBool, std::set<std::string> > _keywords;
+
+    RepoVariablesReplacedUrlList _gpgKeyUrls;
 
     friend Impl * rwcowClone<Impl>( const Impl * rhs );
     /** clone for RWCOW_pointer */
@@ -258,55 +368,123 @@ namespace zypp
   unsigned RepoInfo::defaultPriority()
   { return Impl::defaultPriority; }
 
+  unsigned RepoInfo::noPriority()
+  { return Impl::noPriority; }
+
   void RepoInfo::setPriority( unsigned newval_r )
   { _pimpl->priority = newval_r ? newval_r : Impl::defaultPriority; }
 
 
   bool RepoInfo::gpgCheck() const
-  { return indeterminate(_pimpl->_gpgCheck) ? ZConfig::instance().gpgCheck() : (bool)_pimpl->_gpgCheck; }
+  { return _pimpl->cfgGpgCheck(); }
 
   void RepoInfo::setGpgCheck( TriBool value_r )
-  { _pimpl->_gpgCheck = value_r; }
+  { _pimpl->rawGpgCheck( value_r ); }
 
   void RepoInfo::setGpgCheck( bool value_r ) // deprecated legacy and for squid
   { setGpgCheck( TriBool(value_r) ); }
 
 
   bool RepoInfo::repoGpgCheck() const
+  { return gpgCheck() || _pimpl->cfgRepoGpgCheck(); }
+
+  bool RepoInfo::repoGpgCheckIsMandatory() const
   {
-    if ( ! indeterminate(_pimpl->_repoGpgCheck) )		return _pimpl->_repoGpgCheck;
-    if ( ! indeterminate(ZConfig::instance().repoGpgCheck()) )	return ZConfig::instance().repoGpgCheck();
-    return gpgCheck();	// no preference: follow gpgCheck
+    bool ret = ( gpgCheck() && indeterminate(_pimpl->cfgRepoGpgCheck()) ) || _pimpl->cfgRepoGpgCheck();
+    if ( ret && _pimpl->internalUnsignedConfirmed() )	// relax if unsigned repo was confirmed in the past
+      ret = false;
+    return ret;
   }
 
   void RepoInfo::setRepoGpgCheck( TriBool value_r )
-  { _pimpl->_repoGpgCheck = value_r; }
+  { _pimpl->rawRepoGpgCheck( value_r ); }
 
 
   bool RepoInfo::pkgGpgCheck() const
-  {
-    if ( ! indeterminate(_pimpl->_pkgGpgCheck) )		return _pimpl->_pkgGpgCheck;
-    if ( ! indeterminate(ZConfig::instance().pkgGpgCheck()) )	return ZConfig::instance().pkgGpgCheck();
-    // no preference: follow gpgCheck and check package if repo signature not available or not checked
-    return gpgCheck() && ( !repoGpgCheck() || !(bool)validRepoSignature() );	// !(bool)TriBool ==> false or indeterminate
-  }
+  { return _pimpl->cfgPkgGpgCheck() || ( gpgCheck() && !bool(validRepoSignature())/*enforced*/ ) ; }
+
+  bool RepoInfo::pkgGpgCheckIsMandatory() const
+  { return _pimpl->cfgPkgGpgCheck() || ( gpgCheck() && indeterminate(_pimpl->cfgPkgGpgCheck()) && !bool(validRepoSignature())/*enforced*/ ); }
 
   void RepoInfo::setPkgGpgCheck( TriBool value_r )
-  { _pimpl->_pkgGpgCheck = value_r; }
+  { _pimpl->rawPkgGpgCheck( value_r ); }
+
+
+  void RepoInfo::getRawGpgChecks( TriBool & g_r, TriBool & r_r, TriBool & p_r ) const
+  {
+    g_r = _pimpl->rawGpgCheck();
+    r_r = _pimpl->rawRepoGpgCheck();
+    p_r = _pimpl->rawPkgGpgCheck();
+  }
 
 
   TriBool RepoInfo::validRepoSignature() const
   {
-    TriBool ret = _pimpl->internalValidRepoSignature();
-    // keep indeterminate(=unsigned) but invalidate any signature if !repoGpgCheck
-    if ( !indeterminate(ret) && !repoGpgCheck() )
-      ret = false;
+    TriBool ret( _pimpl->internalValidRepoSignature() );
+    if ( ret && !repoGpgCheck() ) ret = false;	// invalidate any old signature if repoGpgCheck is off
     return ret;
   }
 
   void RepoInfo::setValidRepoSignature( TriBool value_r )
   { _pimpl->internalSetValidRepoSignature( value_r ); }
 
+  ///////////////////////////////////////////////////////////////////
+  namespace
+  {
+    inline bool changeGpgCheckTo( TriBool & lhs, TriBool rhs )
+    { if ( ! sameTriboolState( lhs, rhs ) ) { lhs = rhs; return true; } return false; }
+
+    inline bool changeGpgCheckTo( TriBool ogpg[3], TriBool g, TriBool r, TriBool p )
+    {
+      bool changed = false;
+      if ( changeGpgCheckTo( ogpg[0], g ) ) changed = true;
+      if ( changeGpgCheckTo( ogpg[1], r ) ) changed = true;
+      if ( changeGpgCheckTo( ogpg[2], p ) ) changed = true;
+      return changed;
+    }
+  } // namespace
+  ///////////////////////////////////////////////////////////////////
+  bool RepoInfo::setGpgCheck( GpgCheck mode_r )
+  {
+    TriBool ogpg[3];	// Gpg RepoGpg PkgGpg
+    getRawGpgChecks( ogpg[0], ogpg[1], ogpg[2] );
+
+    bool changed = false;
+    switch ( mode_r )
+    {
+      case GpgCheck::On:
+	changed = changeGpgCheckTo( ogpg, true,          indeterminate, indeterminate );
+	break;
+      case GpgCheck::Strict:
+	changed = changeGpgCheckTo( ogpg, true,          true,          true          );
+	break;
+      case GpgCheck::AllowUnsigned:
+	changed = changeGpgCheckTo( ogpg, true,          false,         false         );
+	break;
+      case GpgCheck::AllowUnsignedRepo:
+	changed = changeGpgCheckTo( ogpg, true,          false,         indeterminate );
+	break;
+      case GpgCheck::AllowUnsignedPackage:
+	changed = changeGpgCheckTo( ogpg, true,          indeterminate, false         );
+	break;
+      case GpgCheck::Default:
+	changed = changeGpgCheckTo( ogpg, indeterminate, indeterminate, indeterminate );
+	break;
+      case GpgCheck::Off:
+	changed = changeGpgCheckTo( ogpg, false,         indeterminate, indeterminate );
+	break;
+      case GpgCheck::indeterminate:	// no change
+	break;
+    }
+
+    if ( changed )
+    {
+      setGpgCheck    ( ogpg[0] );
+      setRepoGpgCheck( ogpg[1] );
+      setPkgGpgCheck ( ogpg[2] );
+    }
+    return changed;
+  }
 
   void RepoInfo::setMirrorListUrl( const Url & url_r )	// Raw
   { _pimpl->_mirrorListUrl.raw() = url_r; _pimpl->_mirrorListForceMetalink = false; }
@@ -314,8 +492,110 @@ namespace zypp
   void  RepoInfo::setMetalinkUrl( const Url & url_r )	// Raw
   { _pimpl->_mirrorListUrl.raw() = url_r; _pimpl->_mirrorListForceMetalink = true; }
 
+  void RepoInfo::setGpgKeyUrls( url_set urls )
+  { _pimpl->gpgKeyUrls().raw().swap( urls ); }
+
   void RepoInfo::setGpgKeyUrl( const Url & url_r )
-  { _pimpl->_gpgKeyUrl.raw() = url_r; }
+  {
+    _pimpl->gpgKeyUrls().raw().clear();
+    _pimpl->gpgKeyUrls().raw().push_back( url_r );
+  }
+
+  Pathname RepoInfo::provideKey(const std::string &keyID_r, const Pathname &targetDirectory_r)
+  {
+    MIL << "Check for " << keyID_r << " at " << targetDirectory_r << endl;
+    std::string keyIDStr( keyID_r.size() > 8 ? keyID_r.substr( keyID_r.size()-8 ) : keyID_r );	// print short ID in Jobreports
+    filesystem::TmpDir tmpKeyRingDir;
+    KeyRing tempKeyRing(tmpKeyRingDir.path());
+
+    // translator: %1% is a gpg key ID like 3DBDC284
+    //             %2% is a cache directories path
+    JobReport::info( str::Format(_("Looking for gpg key ID %1% in cache %2%.") ) % keyIDStr % targetDirectory_r );
+    filesystem::dirForEach(targetDirectory_r,
+                           StrMatcher(".key", Match::STRINGEND),
+                           [&tempKeyRing]( const Pathname & dir_r, const std::string & str_r ){
+      try {
+
+        // deprecate a month old keys
+        PathInfo fileInfo ( dir_r/str_r );
+        if ( Date::now() - fileInfo.mtime() > Date::month ) {
+          //if unlink fails, the file will be overriden in the next step, no need
+          //to show a error
+          filesystem::unlink( dir_r/str_r );
+        } else {
+          tempKeyRing.multiKeyImport(dir_r/str_r, true);
+        }
+      } catch (const KeyRingException& e) {
+        ZYPP_CAUGHT(e);
+        ERR << "Error importing cached key from file '"<<dir_r/str_r<<"'."<<endl;
+      }
+      return true;
+    });
+
+    // no key in the cache is what we are looking for, lets download
+    // all keys specified in gpgkey= entries
+    if ( !tempKeyRing.isKeyTrusted(keyID_r) ) {
+      if ( ! gpgKeyUrlsEmpty() ) {
+	// translator: %1% is a gpg key ID like 3DBDC284
+	//             %2% is a repositories name
+	JobReport::info( str::Format(_("Looking for gpg key ID %1% in repository %2%.") ) % keyIDStr % asUserString() );
+	for ( const Url &url : gpgKeyUrls() ) {
+	  try {
+	    JobReport::info( "  gpgkey=" + url.asString() );
+	    ManagedFile f = MediaSetAccess::provideOptionalFileFromUrl( url );
+	    if ( f->empty() )
+	      continue;
+
+	    PublicKey key(f);
+	    if ( !key.isValid() )
+	      continue;
+
+	    // import all keys into our temporary keyring
+	    tempKeyRing.multiKeyImport(f, true);
+
+	  } catch ( const std::exception & e ) {
+	    //ignore and continue to next url
+	    ZYPP_CAUGHT(e);
+	    MIL << "Key import from url:'"<<url<<"' failed." << endl;
+	  }
+	}
+      }
+      else {
+	// translator: %1% is a repositories name
+	JobReport::info( str::Format(_("Repository %1% does not define additional 'gpgkey=' URLs.") ) % asUserString() );
+      }
+    }
+
+    filesystem::assert_dir( targetDirectory_r );
+
+    //now write all keys into their own files in cache, override existing ones to always have
+    //up to date key data
+    for ( const auto & key: tempKeyRing.trustedPublicKeyData()) {
+      MIL << "KEY ID in KEYRING: " << key.id() << endl;
+
+      Pathname keyFile = targetDirectory_r/(str::Format("%1%.key") % key.rpmName()).asString();
+
+      std::ofstream fout( keyFile.c_str(), std::ios_base::out | std::ios_base::trunc );
+
+      if (!fout)
+        ZYPP_THROW(Exception(str::form("Cannot open file %s",keyFile.c_str())));
+
+      tempKeyRing.dumpTrustedPublicKey( key.id(), fout );
+    }
+
+    // key is STILL not known, we give up
+    if ( !tempKeyRing.isKeyTrusted(keyID_r) ) {
+      return Pathname();
+    }
+
+    PublicKeyData keyData( tempKeyRing.trustedPublicKeyData( keyID_r ) );
+    if ( !keyData ) {
+      ERR << "Error when exporting key from temporary keychain." << endl;
+      return Pathname();
+    }
+
+    return targetDirectory_r/(str::Format("%1%.key") % keyData.rpmName()).asString();
+  }
 
   void RepoInfo::addBaseUrl( const Url & url_r )
   {
@@ -331,6 +611,9 @@ namespace zypp
     _pimpl->baseUrls().raw().push_back( url_r );
   }
 
+  void RepoInfo::setBaseUrls( url_set urls )
+  { _pimpl->baseUrls().raw().swap( urls ); }
+
   void RepoInfo::setPath( const Pathname &path )
   { _pimpl->path = path; }
 
@@ -342,10 +625,10 @@ namespace zypp
 
 
   void RepoInfo::setMetadataPath( const Pathname &path )
-  { _pimpl->metadatapath = path; }
+  { _pimpl->metadataPath( path ); }
 
   void RepoInfo::setPackagesPath( const Pathname &path )
-  { _pimpl->packagespath = path; }
+  { _pimpl->packagesPath( path ); }
 
   void RepoInfo::setKeepPackages( bool keep )
   { _pimpl->keeppackages = keep; }
@@ -360,10 +643,13 @@ namespace zypp
   { return indeterminate(_pimpl->keeppackages) ? false : (bool)_pimpl->keeppackages; }
 
   Pathname RepoInfo::metadataPath() const
-  { return _pimpl->metadatapath; }
+  { return _pimpl->metadataPath(); }
 
   Pathname RepoInfo::packagesPath() const
-  { return _pimpl->packagespath; }
+  { return _pimpl->packagesPath(); }
+
+  bool RepoInfo::usesAutoMethadataPaths() const
+  { return _pimpl->usesAutoMethadataPaths(); }
 
   repo::RepoType RepoInfo::type() const
   { return _pimpl->type; }
@@ -374,11 +660,23 @@ namespace zypp
   Url RepoInfo::rawMirrorListUrl() const		// Raw
   { return _pimpl->_mirrorListUrl.raw(); }
 
+  bool RepoInfo::gpgKeyUrlsEmpty() const
+  { return _pimpl->gpgKeyUrls().empty(); }
+
+  RepoInfo::urls_size_type RepoInfo::gpgKeyUrlsSize() const
+  { return _pimpl->gpgKeyUrls().size(); }
+
+  RepoInfo::url_set RepoInfo::gpgKeyUrls() const	// Variables replaced!
+  { return _pimpl->gpgKeyUrls().transformed(); }
+
+  RepoInfo::url_set RepoInfo::rawGpgKeyUrls() const	// Raw
+  { return _pimpl->gpgKeyUrls().raw(); }
+
   Url RepoInfo::gpgKeyUrl() const			// Variables replaced!
-  { return _pimpl->_gpgKeyUrl.transformed(); }
+  { return( _pimpl->gpgKeyUrls().empty() ? Url() : *_pimpl->gpgKeyUrls().transformedBegin() ); }
 
   Url RepoInfo::rawGpgKeyUrl() const			// Raw
-  {  return _pimpl->_gpgKeyUrl.raw(); }
+  { return( _pimpl->gpgKeyUrls().empty() ? Url() : *_pimpl->gpgKeyUrls().rawBegin() ) ; }
 
   RepoInfo::url_set RepoInfo::baseUrls() const		// Variables replaced!
   { return _pimpl->baseUrls().transformed(); }
@@ -413,28 +711,34 @@ namespace zypp
   bool RepoInfo::baseUrlSet() const
   { return _pimpl->baseurl2dump(); }
 
+  const std::set<std::string> & RepoInfo::contentKeywords() const
+  { return _pimpl->contentKeywords(); }
 
   void RepoInfo::addContent( const std::string & keyword_r )
   { _pimpl->addContent( keyword_r ); }
 
+  bool RepoInfo::hasContent() const
+  { return _pimpl->hasContent(); }
+
   bool RepoInfo::hasContent( const std::string & keyword_r ) const
   { return _pimpl->hasContent( keyword_r ); }
 
   ///////////////////////////////////////////////////////////////////
 
   bool RepoInfo::hasLicense() const
-  {
-    Pathname licenseTgz( _pimpl->licenseTgz() );
-    return ! licenseTgz.empty() &&  PathInfo(licenseTgz).isFile();
-  }
+  { return hasLicense( std::string() ); }
+
+  bool RepoInfo::hasLicense( const std::string & name_r ) const
+  { return !_pimpl->licenseTgz( name_r ).empty(); }
+
 
   bool RepoInfo::needToAcceptLicense() const
-  {
-    static const std::string noAcceptanceFile = "no-acceptance-needed\n";
-    bool accept = true;
+  { return needToAcceptLicense( std::string() ); }
 
-    Pathname licenseTgz( _pimpl->licenseTgz() );
-    if ( licenseTgz.empty() || ! PathInfo( licenseTgz ).isFile() )
+  bool RepoInfo::needToAcceptLicense( const std::string & name_r ) const
+  {
+    const Pathname & licenseTgz( _pimpl->licenseTgz( name_r ) );
+    if ( licenseTgz.empty() )
       return false;     // no licenses at all
 
     ExternalProgram::Arguments cmd;
@@ -443,8 +747,10 @@ namespace zypp
     cmd.push_back( "-z" );
     cmd.push_back( "-f" );
     cmd.push_back( licenseTgz.asString() );
-
     ExternalProgram prog( cmd, ExternalProgram::Stderr_To_Stdout );
+
+    bool accept = true;
+    static const std::string noAcceptanceFile = "no-acceptance-needed\n";
     for ( std::string output( prog.receiveLine() ); output.length(); output = prog.receiveLine() )
     {
       if ( output == noAcceptanceFile )
@@ -452,24 +758,28 @@ namespace zypp
         accept = false;
       }
     }
-    MIL << "License for " << name() << " has to be accepted: " << (accept?"true":"false" ) << endl;
+    prog.close();
+    MIL << "License(" << name_r << ") in " << name() << " has to be accepted: " << (accept?"true":"false" ) << endl;
     return accept;
   }
 
+
   std::string RepoInfo::getLicense( const Locale & lang_r )
-  { return const_cast<const RepoInfo *>(this)->getLicense( lang_r );  }
+  { return const_cast<const RepoInfo *>(this)->getLicense( std::string(), lang_r ); }
 
   std::string RepoInfo::getLicense( const Locale & lang_r ) const
+  { return getLicense( std::string(), lang_r ); }
+
+  std::string RepoInfo::getLicense( const std::string & name_r, const Locale & lang_r ) const
   {
-    LocaleSet avlocales( getLicenseLocales() );
+    LocaleSet avlocales( getLicenseLocales( name_r ) );
     if ( avlocales.empty() )
       return std::string();
 
     Locale getLang( Locale::bestMatch( avlocales, lang_r ) );
-    if ( getLang == Locale::noCode
-         && avlocales.find( Locale::noCode ) == avlocales.end() )
+    if ( !getLang && avlocales.find( Locale::noCode ) == avlocales.end() )
     {
-      WAR << "License.tar.gz contains no fallback text! " << *this << endl;
+      WAR << "License(" << name_r << ") in " << name() << " contains no fallback text!" << endl;
       // Using the fist locale instead of returning no text at all.
       // So the user might recognize that there is a license, even if he
       // can't read it.
@@ -478,9 +788,8 @@ namespace zypp
 
     // now extract the license file.
     static const std::string licenseFileFallback( "license.txt" );
-    std::string licenseFile( getLang == Locale::noCode
-                             ? licenseFileFallback
-                             : str::form( "license.%s.txt", getLang.code().c_str() ) );
+    std::string licenseFile( !getLang ? licenseFileFallback
+				      : str::form( "license.%s.txt", getLang.c_str() ) );
 
     ExternalProgram::Arguments cmd;
     cmd.push_back( "tar" );
@@ -488,7 +797,7 @@ namespace zypp
     cmd.push_back( "-z" );
     cmd.push_back( "-O" );
     cmd.push_back( "-f" );
-    cmd.push_back( _pimpl->licenseTgz().asString() ); // if it not exists, avlocales was empty.
+    cmd.push_back( _pimpl->licenseTgz( name_r ).asString() ); // if it not exists, avlocales was empty.
     cmd.push_back( licenseFile );
 
     std::string ret;
@@ -501,10 +810,14 @@ namespace zypp
     return ret;
   }
 
+
   LocaleSet RepoInfo::getLicenseLocales() const
+  { return getLicenseLocales( std::string() ); }
+
+  LocaleSet RepoInfo::getLicenseLocales( const std::string & name_r ) const
   {
-    Pathname licenseTgz( _pimpl->licenseTgz() );
-    if ( licenseTgz.empty() || ! PathInfo( licenseTgz ).isFile() )
+    const Pathname & licenseTgz( _pimpl->licenseTgz( name_r ) );
+    if ( licenseTgz.empty() )
       return LocaleSet();
 
     ExternalProgram::Arguments cmd;
@@ -558,20 +871,24 @@ namespace zypp
 
     // Yes No Default(Y) Default(N)
 #define OUTS(T,B) ( indeterminate(T) ? (std::string("D(")+(B?"Y":"N")+")") : ((bool)T?"Y":"N") )
-    str << "- gpgcheck    : " << OUTS(_pimpl->_gpgCheck,gpgCheck())
-                              << " repo" << OUTS(_pimpl->_repoGpgCheck,repoGpgCheck())
-			      << " sig" << asString( validRepoSignature(), "?", "Y", "N" )
-			      << " pkg" << OUTS(_pimpl->_pkgGpgCheck,pkgGpgCheck())
+    str << "- gpgcheck    : " << OUTS(_pimpl->rawGpgCheck(),gpgCheck())
+                              << " repo" << OUTS(_pimpl->rawRepoGpgCheck(),repoGpgCheck()) << (repoGpgCheckIsMandatory() ? "* ": " " )
+			      << "sig" << asString( validRepoSignature(), "?", "Y", "N" )
+			      << " pkg" << OUTS(_pimpl->rawPkgGpgCheck(),pkgGpgCheck()) << (pkgGpgCheckIsMandatory() ? "* ": " " )
 			      << std::endl;
 #undef OUTS
 
-    strif( "- gpgkey      : ", rawGpgKeyUrl().asString() );
+    for ( const auto & url : _pimpl->gpgKeyUrls().raw() )
+    {
+      str << "- gpgkey      : " << url << std::endl;
+    }
 
     if ( ! indeterminate(_pimpl->keeppackages) )
       str << "- keeppackages: " << keepPackages() << std::endl;
 
     strif( "- service     : ", service() );
     strif( "- targetdistro: ", targetDistribution() );
+    strif( "- filePath:     ", filepath().asString() );
     strif( "- metadataPath: ", metadataPath().asString() );
     strif( "- packagesPath: ", packagesPath().asString() );
 
@@ -604,17 +921,24 @@ namespace zypp
     if ( priority() != defaultPriority() )
       str << "priority=" << priority() << endl;
 
-    if ( ! indeterminate(_pimpl->_gpgCheck) )
-      str << "gpgcheck=" << (_pimpl->_gpgCheck ? "1" : "0") << endl;
+    if ( ! indeterminate(_pimpl->rawGpgCheck()) )
+      str << "gpgcheck=" << (_pimpl->rawGpgCheck() ? "1" : "0") << endl;
 
-    if ( ! indeterminate(_pimpl->_repoGpgCheck) )
-      str << "repo_gpgcheck=" << (_pimpl->_repoGpgCheck ? "1" : "0") << endl;
+    if ( ! indeterminate(_pimpl->rawRepoGpgCheck()) )
+      str << "repo_gpgcheck=" << (_pimpl->rawRepoGpgCheck() ? "1" : "0") << endl;
 
-    if ( ! indeterminate(_pimpl->_pkgGpgCheck) )
-      str << "pkg_gpgcheck=" << (_pimpl->_pkgGpgCheck ? "1" : "0") << endl;
+    if ( ! indeterminate(_pimpl->rawPkgGpgCheck()) )
+      str << "pkg_gpgcheck=" << (_pimpl->rawPkgGpgCheck() ? "1" : "0") << endl;
 
-    if ( ! (rawGpgKeyUrl().asString().empty()) )
-      str << "gpgkey=" << rawGpgKeyUrl() << endl;
+    {
+      std::string indent( "gpgkey=");
+      for ( const auto & url : _pimpl->gpgKeyUrls().raw() )
+      {
+	str << indent << url << endl;
+	if ( indent[0] != ' ' )
+	  indent = "       ";
+      }
+    }
 
     if (!indeterminate(_pimpl->keeppackages))
       str << "keeppackages=" << keepPackages() << endl;
@@ -663,6 +987,23 @@ namespace zypp
     return obj.dumpOn(str);
   }
 
+  std::ostream & operator<<( std::ostream & str, const RepoInfo::GpgCheck & obj )
+  {
+    switch ( obj )
+    {
+#define OUTS( V ) case RepoInfo::V: return str << #V; break
+      OUTS( GpgCheck::On );
+      OUTS( GpgCheck::Strict );
+      OUTS( GpgCheck::AllowUnsigned );
+      OUTS( GpgCheck::AllowUnsignedRepo );
+      OUTS( GpgCheck::AllowUnsignedPackage );
+      OUTS( GpgCheck::Default );
+      OUTS( GpgCheck::Off );
+      OUTS( GpgCheck::indeterminate );
+#undef OUTS
+    }
+    return str << "GpgCheck::UNKNOWN";
+  }
 
   /////////////////////////////////////////////////////////////////
 } // namespace zypp
