@@ -18,6 +18,9 @@
 #include <map>
 #include <algorithm>
 
+#include <solv/solvversion.h>
+
+#include "zypp/base/String.h"
 #include "zypp/base/InputStream.h"
 #include "zypp/base/LogTools.h"
 #include "zypp/base/Gettext.h"
@@ -36,6 +39,7 @@
 #include "zypp/MediaSetAccess.h"
 #include "zypp/ExternalProgram.h"
 #include "zypp/ManagedFile.h"
+#include "zypp/KeyManager.h"
 
 #include "zypp/parser/RepoFileReader.h"
 #include "zypp/parser/ServiceFileReader.h"
@@ -61,6 +65,89 @@ using namespace zypp::repo;
 ///////////////////////////////////////////////////////////////////
 namespace zypp
 {
+
+  ///////////////////////////////////////////////////////////////////
+  namespace env
+  {
+    /** To trigger appdata refresh unconditionally */
+    inline bool ZYPP_PLUGIN_APPDATA_FORCE_COLLECT()
+    {
+      const char * env = getenv("ZYPP_PLUGIN_APPDATA_FORCE_COLLECT");
+      return( env && str::strToBool( env, true ) );
+    }
+  } // namespace env
+  ///////////////////////////////////////////////////////////////////
+
+  ///////////////////////////////////////////////////////////////////
+  namespace
+  {
+    ///////////////////////////////////////////////////////////////////
+    /// \class UrlCredentialExtractor
+    /// \brief Extract credentials in \ref Url authority and store them via \ref CredentialManager.
+    ///
+    /// Lazy init CredentialManager and save collected credentials when
+    /// going out of scope.
+    ///
+    /// Methods return whether a password has been collected/extracted.
+    ///
+    /// \code
+    /// UrlCredentialExtractor( "/rootdir" ).collect( oneUrlOrUrlContainer );
+    /// \endcode
+    /// \code
+    /// {
+    ///   UrlCredentialExtractor extractCredentials;
+    ///   extractCredentials.collect( oneUrlOrUrlContainer );
+    ///   extractCredentials.extract( oneMoreUrlOrUrlContainer );
+    ///   ....
+    /// }
+    /// \endcode
+    ///
+    class UrlCredentialExtractor
+    {
+    public:
+      UrlCredentialExtractor( Pathname & root_r )
+      : _root( root_r )
+      {}
+
+      ~UrlCredentialExtractor()
+      { if ( _cmPtr ) _cmPtr->save(); }
+
+      /** Remember credentials stored in URL authority leaving the password in \a url_r. */
+      bool collect( const Url & url_r )
+      {
+	bool ret = url_r.hasCredentialsInAuthority();
+	if ( ret )
+	{
+	  if ( !_cmPtr ) _cmPtr.reset( new media::CredentialManager( _root ) );
+	  _cmPtr->addUserCred( url_r );
+	}
+	return ret;
+      }
+      /** \overload operating on Url container */
+      template<class TContainer>
+      bool collect( const TContainer & urls_r )
+      {	bool ret = false; for ( const Url & url : urls_r ) { if ( collect( url ) && !ret ) ret = true; } return ret; }
+
+      /** Remember credentials stored in URL authority stripping the passowrd from \a url_r. */
+      bool extract( Url & url_r )
+      {
+	bool ret = collect( url_r );
+	if ( ret )
+	  url_r.setPassword( std::string() );
+	return ret;
+      }
+      /** \overload operating on Url container */
+      template<class TContainer>
+      bool extract( TContainer & urls_r )
+      {	bool ret = false; for ( Url & url : urls_r ) { if ( extract( url ) && !ret ) ret = true; } return ret; }
+
+    private:
+      const Pathname & _root;
+      scoped_ptr<media::CredentialManager> _cmPtr;
+    };
+  } // namespace
+  ///////////////////////////////////////////////////////////////////
+
   ///////////////////////////////////////////////////////////////////
   namespace
   {
@@ -224,7 +311,7 @@ namespace zypp
       bool nonroot( geteuid() != 0 );
       if ( nonroot && ! PathInfo(dir).userMayRX() )
       {
-	JobReport::warning( formatNAC(_("Cannot read repo directory '%1%': Permission denied")) % dir );
+	JobReport::warning( str::Format(_("Cannot read repo directory '%1%': Permission denied")) % dir );
       }
       else
       {
@@ -242,7 +329,7 @@ namespace zypp
 	  {
 	    if ( nonroot && ! PathInfo(*it).userMayR() )
 	    {
-	      JobReport::warning( formatNAC(_("Cannot read repo file '%1%': Permission denied")) % *it );
+	      JobReport::warning( str::Format(_("Cannot read repo file '%1%': Permission denied")) % *it );
 	    }
 	    else
 	    {
@@ -295,6 +382,15 @@ namespace zypp
 
     ////////////////////////////////////////////////////////////////////////////
 
+    ///////////////////////////////////////////////////////////////////
+    namespace
+    {
+      /** Whether repo is not under RM control and provides it's own methadata paths. */
+      inline bool isTmpRepo( const RepoInfo & info_r )
+      { return( info_r.filepath().empty() && info_r.usesAutoMethadataPaths() ); }
+    } // namespace
+    ///////////////////////////////////////////////////////////////////
+
     /**
      * \short Calculates the raw cache path for a repository, this is usually
      * /var/cache/zypp/alias
@@ -302,7 +398,7 @@ namespace zypp
     inline Pathname rawcache_path_for_repoinfo( const RepoManagerOptions &opt, const RepoInfo &info )
     {
       assert_alias(info);
-      return opt.repoRawCachePath / info.escaped_alias();
+      return isTmpRepo( info ) ? info.metadataPath() : opt.repoRawCachePath / info.escaped_alias();
     }
 
     /**
@@ -314,10 +410,7 @@ namespace zypp
      * for example /var/cache/zypp/alias/addondir
      */
     inline Pathname rawproductdata_path_for_repoinfo( const RepoManagerOptions &opt, const RepoInfo &info )
-    {
-      assert_alias(info);
-      return opt.repoRawCachePath / info.escaped_alias() / info.path();
-    }
+    { return rawcache_path_for_repoinfo( opt, info ) / info.path(); }
 
     /**
      * \short Calculates the packages cache path for a repository
@@ -325,16 +418,16 @@ namespace zypp
     inline Pathname packagescache_path_for_repoinfo( const RepoManagerOptions &opt, const RepoInfo &info )
     {
       assert_alias(info);
-      return opt.repoPackagesCachePath / info.escaped_alias();
+      return isTmpRepo( info ) ? info.packagesPath() : opt.repoPackagesCachePath / info.escaped_alias();
     }
 
     /**
      * \short Calculates the solv cache path for a repository
      */
-    inline Pathname solv_path_for_repoinfo( const RepoManagerOptions &opt, const RepoInfo &info)
+    inline Pathname solv_path_for_repoinfo( const RepoManagerOptions &opt, const RepoInfo &info )
     {
       assert_alias(info);
-      return opt.repoSolvCachePath / info.escaped_alias();
+      return isTmpRepo( info ) ? info.metadataPath().dirname() / "%SLV%" : opt.repoSolvCachePath / info.escaped_alias();
     }
 
     ////////////////////////////////////////////////////////////////////////////
@@ -365,13 +458,7 @@ namespace zypp
 
   std::list<RepoInfo> readRepoFile( const Url & repo_file )
   {
-    // no interface to download a specific file, using workaround:
-    //! \todo add MediaManager::provideFile(Url file_url) to easily access any file URLs? (no need for media access id or media_nr)
-    Url url(repo_file);
-    Pathname path(url.getPathName());
-    url.setPathName ("/");
-    MediaSetAccess access(url);
-    Pathname local = access.provideFile(path);
+    ManagedFile local = MediaSetAccess::provideFileFromUrl(repo_file);
 
     DBG << "reading repo file " << repo_file << ", local path: " << local << endl;
 
@@ -445,7 +532,8 @@ namespace zypp
     ~Impl()
     {
       // trigger appdata refresh if some repos change
-      if ( _reposDirty && geteuid() == 0 && ( _options.rootDir.empty() || _options.rootDir == "/" ) )
+      if ( ( _reposDirty || env::ZYPP_PLUGIN_APPDATA_FORCE_COLLECT() )
+	&& geteuid() == 0 && ( _options.rootDir.empty() || _options.rootDir == "/" ) )
       {
 	try {
 	  std::list<Pathname> entries;
@@ -520,6 +608,7 @@ namespace zypp
     void buildCache( const RepoInfo & info, CacheBuildPolicy policy, OPT_PROGRESS );
 
     repo::RepoType probe( const Url & url, const Pathname & path = Pathname() ) const;
+    repo::RepoType probeCache( const Pathname & path_r ) const;
 
     void cleanCacheDirGarbage( OPT_PROGRESS );
 
@@ -703,7 +792,7 @@ namespace zypp
       }
     }
 
-    repo::PluginServices(_options.pluginsPath/"services", ServiceCollector(_services));
+   repo::PluginServices(_options.pluginsPath/"services", ServiceCollector(_services));
   }
 
   ///////////////////////////////////////////////////////////////////
@@ -769,13 +858,13 @@ namespace zypp
       // Cleanup orphanded service repos:
       if ( ! orphanedRepos.empty() )
       {
-	for ( auto & repoInfo : orphanedRepos )
+	for ( const auto & repoInfo : orphanedRepos )
 	{
 	  MIL << "Delete orphaned service repo " << repoInfo.alias() << endl;
 	  // translators: Cleanup a repository previously owned by a meanwhile unknown (deleted) service.
 	  //   %1% = service name
 	  //   %2% = repository name
-	  JobReport::warning( formatNAC(_("Unknown service '%1%': Removing orphaned service repository '%2%'" ))
+	  JobReport::warning( str::Format(_("Unknown service '%1%': Removing orphaned service repository '%2%'"))
 			      % repoInfo.service()
 			      % repoInfo.alias() );
 	  try {
@@ -812,13 +901,13 @@ namespace zypp
     RepoType repokind = info.type();
     // If unknown, probe the local metadata
     if ( repokind == RepoType::NONE )
-      repokind = probe( productdatapath.asUrl() );
+      repokind = probeCache( productdatapath );
 
     RepoStatus status;
     switch ( repokind.toEnum() )
     {
       case RepoType::RPMMD_e :
-        status = RepoStatus( productdatapath/"repodata/repomd.xml");
+        status = RepoStatus( productdatapath/"repodata/repomd.xml") && RepoStatus( mediarootpath/"media.1/media" );
 	break;
 
       case RepoType::YAST2_e :
@@ -846,7 +935,7 @@ namespace zypp
     RepoType repokind = info.type();
     if ( repokind.toEnum() == RepoType::NONE_e )
       // unknown, probe the local metadata
-      repokind = probe( productdatapath.asUrl() );
+      repokind = probeCache( productdatapath );
     // if still unknown, just return
     if (repokind == RepoType::NONE_e)
       return;
@@ -881,33 +970,37 @@ namespace zypp
     assert_alias(info);
     try
     {
-      MIL << "Going to try to check whether refresh is needed for " << url << endl;
+      MIL << "Going to try to check whether refresh is needed for " << url << " (" << info.type() << ")" << endl;
 
       // first check old (cached) metadata
       Pathname mediarootpath = rawcache_path_for_repoinfo( _options, info );
       filesystem::assert_dir( mediarootpath );
       RepoStatus oldstatus = metadataStatus( info );
-
       if ( oldstatus.empty() )
       {
         MIL << "No cached metadata, going to refresh" << endl;
         return REFRESH_NEEDED;
       }
 
+      if ( url.schemeIsVolatile() )
       {
-        if ( url.schemeIsVolatile() )
-	{
-	  MIL << "never refresh CD/DVD" << endl;
-          return REPO_UP_TO_DATE;
-	}
-	if ( url.schemeIsLocal() )
-	{
-	  policy = RefreshIfNeededIgnoreDelay;
-	}
+	MIL << "Never refresh CD/DVD" << endl;
+	return REPO_UP_TO_DATE;
+      }
+
+      if ( policy == RefreshForced )
+      {
+	MIL << "Forced refresh!" << endl;
+	return REFRESH_NEEDED;
+      }
+
+      if ( url.schemeIsLocal() )
+      {
+	policy = RefreshIfNeededIgnoreDelay;
       }
 
       // now we've got the old (cached) status, we can decide repo.refresh.delay
-      if (policy != RefreshForced && policy != RefreshIfNeededIgnoreDelay)
+      if ( policy != RefreshIfNeededIgnoreDelay )
       {
         // difference in seconds
         double diff = difftime(
@@ -969,27 +1062,17 @@ namespace zypp
       }
 
       // check status
-      bool refresh = false;
       if ( oldstatus == newstatus )
       {
 	MIL << "repo has not changed" << endl;
-	if ( policy == RefreshForced )
-	{
-	  MIL << "refresh set to forced" << endl;
-	  refresh = true;
-	}
+	touchIndexFile( info );
+	return REPO_UP_TO_DATE;
       }
-      else
+      else // includes newstatus.empty() if e.g. repo format changed
       {
 	MIL << "repo has changed, going to refresh" << endl;
-	refresh = true;
+	return REFRESH_NEEDED;
       }
-
-      if (!refresh)
-	touchIndexFile(info);
-
-      return refresh ? REFRESH_NEEDED : REPO_UP_TO_DATE;
-
     }
     catch ( const Exception &e )
     {
@@ -1008,13 +1091,12 @@ namespace zypp
     assert_urls(info);
 
     // we will throw this later if no URL checks out fine
-    RepoException rexception( info, _PL("Valid metadata not found at specified URL",
+    RepoException rexception( info, PL_("Valid metadata not found at specified URL",
 					"Valid metadata not found at specified URLs",
 					info.baseUrlsSize() ) );
 
     // Suppress (interactive) media::MediaChangeReport if we in have multiple basurls (>1)
     media::ScopedDisableMediaChangeReport guard( info.baseUrlsSize() > 1 );
-
     // try urls one by one
     for ( RepoInfo::urls_const_iterator it = info.baseUrlsBegin(); it != info.baseUrlsEnd(); ++it )
     {
@@ -1029,16 +1111,15 @@ namespace zypp
 
         MIL << "Going to refresh metadata from " << url << endl;
 
+	// bsc#1048315: Always re-probe in case of repo format change.
+	// TODO: Would be sufficient to verify the type and re-probe
+	// if verification failed (or type is RepoType::NONE)
         repo::RepoType repokind = info.type();
-
-        // if the type is unknown, try probing.
-	if ( repokind == RepoType::NONE )
 	{
-	  // unknown, probe it
-	  repokind = probe( *it, info.path() );
-
-	  if (repokind.toEnum() != RepoType::NONE_e)
+	  repo::RepoType probed = probe( *it, info.path() );
+	  if ( repokind != probed )
 	  {
+	    repokind = probed;
 	    // Adjust the probed type in RepoInfo
 	    info.setProbedType( repokind ); // lazy init!
 	    //save probed type only for repos in system
@@ -1115,7 +1196,8 @@ namespace zypp
         // ok we have the metadata, now exchange
         // the contents
 	filesystem::exchange( tmpdir.path(), mediarootpath );
-	reposManip();	// remember to trigger appdata refresh
+	if ( ! isTmpRepo( info ) )
+	  reposManip();	// remember to trigger appdata refresh
 
         // we are done.
         return;
@@ -1130,6 +1212,9 @@ namespace zypp
         // cause of the problem of the first URL remembered
         if (it == info.baseUrlsBegin())
           rexception.remember(e);
+	else
+	  rexception.addHistory(  e.asUserString() );
+
       }
     } // for every url
     ERR << "No more urls..." << endl;
@@ -1188,8 +1273,14 @@ namespace zypp
       if ( cache_status == raw_metadata_status )
       {
         MIL << info.alias() << " cache is up to date with metadata." << endl;
-        if ( policy == BuildIfNeeded ) {
-          return;
+        if ( policy == BuildIfNeeded )
+	{
+	  // On the fly add missing solv.idx files for bash completion.
+	  const Pathname & base = solv_path_for_repoinfo( _options, info);
+	  if ( ! PathInfo(base/"solv.idx").isExist() )
+	    sat::updateSolvFileIndex( base/"solv" );
+
+	  return;
         }
         else {
           MIL << info.alias() << " cache rebuild is forced" << endl;
@@ -1235,7 +1326,7 @@ namespace zypp
     {
       case RepoType::NONE_e:
         // unknown, probe the local metadata
-        repokind = probe( productdatapath.asUrl() );
+        repokind = probeCache( productdatapath );
       break;
       default:
       break;
@@ -1254,15 +1345,16 @@ namespace zypp
         scoped_ptr<MediaMounter> forPlainDirs;
 
         ExternalProgram::Arguments cmd;
-        cmd.push_back( "repo2solv.sh" );
+        cmd.push_back( PathInfo( "/usr/bin/repo2solv" ).isFile() ? "repo2solv" : "repo2solv.sh" );
         // repo2solv expects -o as 1st arg!
         cmd.push_back( "-o" );
         cmd.push_back( solvfile.asString() );
 	cmd.push_back( "-X" );	// autogenerate pattern from pattern-package
+        cmd.push_back( "-A" );	// autogenerate application pseudo packages
 
         if ( repokind == RepoType::RPMPLAINDIR )
         {
-          forPlainDirs.reset( new MediaMounter( *info.baseUrlsBegin() ) );
+          forPlainDirs.reset( new MediaMounter( info.url() ) );
           // recusive for plaindir as 2nd arg!
           cmd.push_back( "-R" );
           // FIXME this does only work form dir: URLs
@@ -1308,6 +1400,13 @@ namespace zypp
 
   ////////////////////////////////////////////////////////////////////////////
 
+
+  /** Probe the metadata type of a repository located at \c url.
+   * Urls here may be rewritten by \ref MediaSetAccess to reflect the correct media number.
+   *
+   * \note Metadata in local cache directories must be probed using \ref probeCache as
+   * a cache path must not be rewritten (bnc#946129)
+   */
   repo::RepoType RepoManager::Impl::probe( const Url & url, const Pathname & path  ) const
   {
     MIL << "going to probe the repo type at " << url << " (" << path << ")" << endl;
@@ -1392,6 +1491,28 @@ namespace zypp
     return repo::RepoType::NONE;
   }
 
+  /** Probe Metadata in a local cache directory
+   *
+   * \note Metadata in local cache directories must not be probed using \ref probe as
+   * a cache path must not be rewritten (bnc#946129)
+   */
+  repo::RepoType RepoManager::Impl::probeCache( const Pathname & path_r ) const
+  {
+    MIL << "going to probe the cached repo at " << path_r << endl;
+
+    repo::RepoType ret = repo::RepoType::NONE;
+
+    if ( PathInfo(path_r/"/repodata/repomd.xml").isFile() )
+    { ret = repo::RepoType::RPMMD; }
+    else if ( PathInfo(path_r/"/content").isFile() )
+    { ret = repo::RepoType::YAST2; }
+    else if ( PathInfo(path_r).isDir() )
+    { ret = repo::RepoType::RPMPLAINDIR; }
+
+    MIL << "Probed cached type " << ret << " at " << path_r << endl;
+    return ret;
+  }
+
   ////////////////////////////////////////////////////////////////////////////
 
   void RepoManager::Impl::cleanCacheDirGarbage( const ProgressData::ReceiverFnc & progressrcv )
@@ -1468,19 +1589,13 @@ namespace zypp
     {
       Repository repo = sat::Pool::instance().addRepoSolv( solvfile, info );
       // test toolversion in order to rebuild solv file in case
-      // it was written by an old libsolv-tool parser.
-      //
-      // Known version strings used:
-      //  - <no string>
-      //  - "1.0"
-      //
-      sat::LookupRepoAttr toolversion( sat::SolvAttr::repositoryToolVersion, repo );
-      if ( toolversion.begin().asString().empty() )
+      // it was written by a different libsolv-tool parser.
+      const std::string & toolversion( sat::LookupRepoAttr( sat::SolvAttr::repositoryToolVersion, repo ).begin().asString() );
+      if ( toolversion != LIBSOLV_TOOLVERSION )
       {
         repo.eraseFromPool();
-        ZYPP_THROW(Exception("Solv-file was created by old parser."));
+        ZYPP_THROW(Exception(str::Str() << "Solv-file was created by '"<<toolversion<<"'-parser (want "<<LIBSOLV_TOOLVERSION<<")."));
       }
-      // else: up-to-date (or even newer).
     }
     catch ( const Exception & exp )
     {
@@ -1515,16 +1630,13 @@ namespace zypp
     if ( _options.probe )
     {
       DBG << "unknown repository type, probing" << endl;
+      assert_urls(tosave);
 
-      RepoType probedtype;
-      probedtype = probe( *tosave.baseUrlsBegin(), info.path() );
-      if ( tosave.baseUrlsSize() > 0 )
-      {
-        if ( probedtype == RepoType::NONE )
-          ZYPP_THROW(RepoUnknownTypeException(info));
-        else
-          tosave.setType(probedtype);
-      }
+      RepoType probedtype( probe( tosave.url(), info.path() ) );
+      if ( probedtype == RepoType::NONE )
+	ZYPP_THROW(RepoUnknownTypeException(info));
+      else
+	tosave.setType(probedtype);
     }
 
     progress.set(50);
@@ -1546,40 +1658,24 @@ namespace zypp
 
     tosave.dumpAsIniOn(file);
     tosave.setFilepath(repofile);
-    tosave.setMetadataPath( metadataPath( tosave ) );
-    tosave.setPackagesPath( packagesPath( tosave ) );
+    tosave.setMetadataPath( rawcache_path_for_repoinfo( _options, tosave ) );
+    tosave.setPackagesPath( packagescache_path_for_repoinfo( _options, tosave ) );
     {
-      // We chould fix the API as we must injet those paths
+      // We should fix the API as we must inject those paths
       // into the repoinfo in order to keep it usable.
       RepoInfo & oinfo( const_cast<RepoInfo &>(info) );
-      oinfo.setMetadataPath( metadataPath( tosave ) );
-      oinfo.setPackagesPath( packagesPath( tosave ) );
+      oinfo.setFilepath(repofile);
+      oinfo.setMetadataPath( rawcache_path_for_repoinfo( _options, tosave ) );
+      oinfo.setPackagesPath( packagescache_path_for_repoinfo( _options, tosave ) );
     }
     reposManip().insert(tosave);
 
     progress.set(90);
 
     // check for credentials in Urls
-    bool havePasswords = false;
-    for_( urlit, tosave.baseUrlsBegin(), tosave.baseUrlsEnd() )
-      if ( urlit->hasCredentialsInAuthority() )
-      {
-        havePasswords = true;
-        break;
-      }
-    // save the credentials
-    if ( havePasswords )
-    {
-      media::CredentialManager cm(
-          media::CredManagerOptions(_options.rootDir) );
+    UrlCredentialExtractor( _options.rootDir ).collect( tosave.baseUrls() );
 
-      for_(urlit, tosave.baseUrlsBegin(), tosave.baseUrlsEnd())
-        if (urlit->hasCredentialsInAuthority())
-          //! \todo use a method calling UI callbacks to ask where to save creds?
-          cm.saveInUser(media::AuthData(*urlit));
-    }
-
-    HistoryLog().addRepository(tosave);
+    HistoryLog(_options.rootDir).addRepository(tosave);
 
     progress.toMax();
     MIL << "done" << endl;
@@ -1631,8 +1727,10 @@ namespace zypp
           ++it )
     {
       MIL << "Saving " << (*it).alias() << endl;
-      it->setFilepath(repofile.asString());
       it->dumpAsIniOn(file);
+      it->setFilepath(repofile);
+      it->setMetadataPath( rawcache_path_for_repoinfo( _options, *it ) );
+      it->setPackagesPath( packagescache_path_for_repoinfo( _options, *it ) );
       reposManip().insert(*it);
 
       HistoryLog(_options.rootDir).addRepository(*it);
@@ -1673,10 +1771,12 @@ namespace zypp
       {
         // figure how many repos are there in the file:
         std::list<RepoInfo> filerepos = repositories_in_file(todelete.filepath());
-        if ( (filerepos.size() == 1) && ( filerepos.front().alias() == todelete.alias() ) )
+        if ( filerepos.size() == 0	// bsc#984494: file may have already been deleted
+	  ||(filerepos.size() == 1 && filerepos.front().alias() == todelete.alias() ) )
         {
-          // easy, only this one, just delete the file
-          if ( filesystem::unlink(todelete.filepath()) != 0 )
+          // easy: file does not exist, contains no or only the repo to delete: delete the file
+	  int ret = filesystem::unlink( todelete.filepath() );
+          if ( ! ( ret == 0 || ret == ENOENT ) )
           {
             // TranslatorExplanation '%s' is a filename
             ZYPP_THROW(RepoException( todelete, str::form( _("Can't delete '%s'"), todelete.filepath().c_str() )));
@@ -1776,9 +1876,29 @@ namespace zypp
             newinfo.dumpAsIniOn(file);
       }
 
+      if ( toedit.enabled() && !newinfo.enabled() )
+      {
+	// On the fly remove solv.idx files for bash completion if a repo gets disabled.
+	const Pathname & solvidx = solv_path_for_repoinfo(_options, newinfo)/"solv.idx";
+	if ( PathInfo(solvidx).isExist() )
+	  filesystem::unlink( solvidx );
+      }
+
       newinfo.setFilepath(toedit.filepath());
+      newinfo.setMetadataPath( rawcache_path_for_repoinfo( _options, newinfo ) );
+      newinfo.setPackagesPath( packagescache_path_for_repoinfo( _options, newinfo ) );
+      {
+	// We should fix the API as we must inject those paths
+	// into the repoinfo in order to keep it usable.
+	RepoInfo & oinfo( const_cast<RepoInfo &>(newinfo_r) );
+	oinfo.setFilepath(toedit.filepath());
+	oinfo.setMetadataPath( rawcache_path_for_repoinfo( _options, newinfo ) );
+	oinfo.setPackagesPath( packagescache_path_for_repoinfo( _options, newinfo ) );
+      }
       reposManip().erase(toedit);
       reposManip().insert(newinfo);
+      // check for credentials in Urls
+      UrlCredentialExtractor( _options.rootDir ).collect( newinfo.baseUrls() );
       HistoryLog(_options.rootDir).modifyRepository(toedit, newinfo);
       MIL << "repo " << alias << " modified" << endl;
     }
@@ -1832,15 +1952,8 @@ namespace zypp
     saveService( toSave );
     _services.insert( toSave );
 
-    // check for credentials in Url (username:password, not ?credentials param)
-    if ( toSave.url().hasCredentialsInAuthority() )
-    {
-      media::CredentialManager cm(
-          media::CredManagerOptions(_options.rootDir) );
-
-      //! \todo use a method calling UI callbacks to ask where to save creds?
-      cm.saveInUser(media::AuthData(toSave.url()));
-    }
+    // check for credentials in Url
+    UrlCredentialExtractor( _options.rootDir ).collect( toSave.url() );
 
     MIL << "added service " << toSave.alias() << endl;
   }
@@ -1926,13 +2039,34 @@ namespace zypp
     ServiceInfo service( getService( alias ) );
     assert_alias( service );
     assert_url( service );
+    MIL << "Going to refresh service '" << service.alias() <<  "', url: " << service.url() << ", opts: " << options_r << endl;
+
+    if ( service.ttl() && !( options_r.testFlag( RefreshService_forceRefresh) || options_r.testFlag( RefreshService_restoreStatus ) ) )
+    {
+      // Service defines a TTL; maybe we can re-use existing data without refresh.
+      Date lrf = service.lrf();
+      if ( lrf )
+      {
+	Date now( Date::now() );
+	if ( lrf <= now )
+	{
+	  if ( (lrf+=service.ttl()) > now ) // lrf+= !
+	  {
+	    MIL << "Skip: '" << service.alias() << "' metadata valid until " << lrf << endl;
+	    return;
+	  }
+	}
+	else
+	  WAR << "Force: '" << service.alias() << "' metadata last refresh in the future: " << lrf << endl;
+      }
+    }
+
     // NOTE: It might be necessary to modify and rewrite the service info.
     // Either when probing the type, or when adjusting the repositories
     // enable/disable state.:
     bool serviceModified = false;
-    MIL << "Going to refresh service '" << service.alias() << "', url: "<< service.url() << ", opts: " << options_r << endl;
 
-    //! \todo add callbacks for apps (start, end, repo removed, repo added, repo changed)
+    //! \todo add callbacks for apps (start, end, repo removed, repo added, repo changed)?
 
     // if the type is unknown, try probing.
     if ( service.type() == repo::ServiceType::NONE )
@@ -1954,6 +2088,7 @@ namespace zypp
     DBG << "ServicesTargetDistro: " << servicesTargetDistro << endl;
 
     // parse it
+    Date::Duration origTtl = service.ttl();	// FIXME Ugly hack: const service.ttl modified when parsing
     RepoCollector collector(servicesTargetDistro);
     // FIXME Ugly hack: ServiceRepos may throw ServicePluginInformalException
     // which is actually a notification. Using an exception for this
@@ -1961,7 +2096,13 @@ namespace zypp
     // and in zypper.
     std::pair<DefaultIntegral<bool,false>, repo::ServicePluginInformalException> uglyHack;
     try {
-      ServiceRepos repos(service, bind( &RepoCollector::collect, &collector, _1 ));
+      // FIXME bsc#1080693: Shortcoming of (plugin)services (and repos as well) is that they
+      // are not aware of the RepoManagers rootDir. The service url, as created in known_services,
+      // contains the full path to the script. The script however has to be executed chrooted.
+      // Repos would need to know the RepoMangers rootDir to use the correct vars.d to replace
+      // repos variables. Until RepoInfoBase is aware if the rootDir, we need to explicitly pass it
+      // to ServiceRepos.
+      ServiceRepos( _options.rootDir, service, bind( &RepoCollector::collect, &collector, _1 ) );
     }
     catch ( const repo::ServicePluginInformalException & e )
     {
@@ -1969,7 +2110,12 @@ namespace zypp
       uglyHack.first = true;
       uglyHack.second = e;
     }
-
+    if ( service.ttl() != origTtl )	// repoindex.xml changed ttl
+    {
+      if ( !service.ttl() )
+	service.setLrf( Date() );	// don't need lrf when zero ttl
+      serviceModified = true;
+    }
     ////////////////////////////////////////////////////////////////////////////
     // On the fly remember the new repo states as defined the reopoindex.xml.
     // Move into ServiceInfo later.
@@ -1980,34 +2126,41 @@ namespace zypp
     {
       // First of all: Prepend service alias:
       it->setAlias( str::form( "%s:%s", service.alias().c_str(), it->alias().c_str() ) );
-      // set refrence to the parent service
+      // set reference to the parent service
       it->setService( service.alias() );
 
       // remember the new parsed repo state
       newRepoStates[it->alias()] = *it;
 
-      // if the repo url was not set by the repoindex parser, set service's url
-      Url url;
-      if ( it->baseUrlsEmpty() )
-        url = service.rawUrl();
-      else
-      {
-        // service repo can contain only one URL now, so no need to iterate.
-        url = it->rawUrl();	// raw!
-      }
-
-      // libzypp currently has problem with separate url + path handling
-      // so just append the path to the baseurl
+      // - If the repo url was not set by the repoindex parser, set service's url.
+      // - Libzypp currently has problem with separate url + path handling so just
+      //   append a path, if set, to the baseurls
+      // - Credentials in the url authority will be extracted later, either if the
+      //   repository is added or if we check for changed urls.
+      Pathname path;
       if ( !it->path().empty() )
       {
-        Pathname path(url.getPathName());
-        path /= it->path();
-        url.setPathName( path.asString() );
-        it->setPath("");
+	if ( it->path() != "/" )
+	  path = it->path();
+	it->setPath("");
       }
 
-      // save the url
-      it->setBaseUrl( url );
+      if ( it->baseUrlsEmpty() )
+      {
+	Url url( service.rawUrl() );
+	if ( !path.empty() )
+	  url.setPathName( url.getPathName() / path );
+	it->setBaseUrl( std::move(url) );
+      }
+      else if ( !path.empty() )
+      {
+	RepoInfo::url_set urls( it->rawBaseUrls() );
+	for ( Url & url : urls )
+	{
+	  url.setPathName( url.getPathName() / path );
+	}
+	it->setBaseUrls( std::move(urls) );
+      }
     }
 
     ////////////////////////////////////////////////////////////////////////////
@@ -2043,7 +2196,8 @@ namespace zypp
     }
 
     ////////////////////////////////////////////////////////////////////////////
-    // create missing repositories and modify exising ones if needed...
+    // create missing repositories and modify existing ones if needed...
+    UrlCredentialExtractor urlCredentialExtractor( _options.rootDir );	// To collect any credentials stored in repo URLs
     for_( it, collector.repos.begin(), collector.repos.end() )
     {
       // User explicitly requested the repo being enabled?
@@ -2168,13 +2322,37 @@ namespace zypp
 	}
 
         // changed url?
-        // service repo can contain only one URL now, so no need to iterate.
-        if ( oldRepo->rawUrl() != it->rawUrl() )
         {
-          DBG << "Service repo " << it->alias() << " gets new URL " << it->rawUrl() << endl;
-          oldRepo->setBaseUrl( it->rawUrl() );
-          oldRepoModified = true;
-        }
+	  RepoInfo::url_set newUrls( it->rawBaseUrls() );
+	  urlCredentialExtractor.extract( newUrls );	// Extract! to prevent passwds from disturbing the comparison below
+	  if ( oldRepo->rawBaseUrls() != newUrls )
+	  {
+	    DBG << "Service repo " << it->alias() << " gets new URLs " << newUrls << endl;
+	    oldRepo->setBaseUrls( std::move(newUrls) );
+	    oldRepoModified = true;
+	  }
+	}
+
+        // changed gpg check settings?
+	// ATM only plugin services can set GPG values.
+	if ( service.type() == ServiceType::PLUGIN )
+	{
+	  TriBool ogpg[3];	// Gpg RepoGpg PkgGpg
+	  TriBool ngpg[3];
+	  oldRepo->getRawGpgChecks( ogpg[0], ogpg[1], ogpg[2] );
+	  it->     getRawGpgChecks( ngpg[0], ngpg[1], ngpg[2] );
+#define Z_CHKGPG(I,N)										\
+	  if ( ! sameTriboolState( ogpg[I], ngpg[I] ) )						\
+	  {											\
+	    DBG << "Service repo " << it->alias() << " gets new "#N"Check " << ngpg[I] << endl;	\
+	    oldRepo->set##N##Check( ngpg[I] );							\
+	    oldRepoModified = true;								\
+	  }
+	  Z_CHKGPG( 0, Gpg );
+	  Z_CHKGPG( 1, RepoGpg );
+	  Z_CHKGPG( 2, PkgGpg );
+#undef Z_CHKGPG
+	}
 
         // save if modified:
         if ( oldRepoModified )
@@ -2200,10 +2378,19 @@ namespace zypp
 
     ////////////////////////////////////////////////////////////////////////////
     // save service if modified: (unless a plugin service)
-    if ( serviceModified && service.type() != ServiceType::PLUGIN )
+    if ( service.type() != ServiceType::PLUGIN )
     {
-      // write out modified service file.
-      modifyService( service.alias(), service );
+      if ( service.ttl() )
+      {
+	service.setLrf( Date::now() );	// remember last refresh
+	serviceModified =  true;	// or use a cookie file
+      }
+
+      if ( serviceModified )
+      {
+	// write out modified service file.
+	modifyService( service.alias(), service );
+      }
     }
 
     if ( uglyHack.first )
@@ -2252,6 +2439,9 @@ namespace zypp
 
     _services.erase(oldAlias);
     _services.insert(service);
+    // check for credentials in Urls
+    UrlCredentialExtractor( _options.rootDir ).collect( service.url() );
+
 
     // changed properties affecting also repositories
     if ( oldAlias != service.alias()			// changed alias
