@@ -25,11 +25,13 @@
 #include "zypp/ZConfig.h"
 #include "zypp/RepoInfo.h"
 #include "zypp/RepoManager.h"
+#include "zypp/SrcPackage.h"
 
 #include "zypp/ZYppFactory.h"
 #include "zypp/Target.h"
 #include "zypp/target/rpm/RpmDb.h"
 #include "zypp/FileChecker.h"
+#include "zypp/target/rpm/RpmHeader.h"
 
 using std::endl;
 
@@ -39,6 +41,27 @@ namespace zypp
   ///////////////////////////////////////////////////////////////////
   namespace repo
   {
+    ///////////////////////////////////////////////////////////////////
+    /// \class RpmSigCheckException
+    /// \brief Exception thrown by \ref PackageProviderImpl::rpmSigFileChecker
+    ///////////////////////////////////////////////////////////////////
+    class RpmSigCheckException : public FileCheckException
+    {
+    public:
+      RpmSigCheckException( repo::DownloadResolvableReport::Action action_r, std::string msg_r = "RpmSigCheckException" )
+      : FileCheckException( std::move(msg_r) )
+      , _action( std::move(action_r) )
+      {}
+
+      /** Users final decision how to proceed */
+      const repo::DownloadResolvableReport::Action & action() const
+      { return _action; }
+
+    private:
+      repo::DownloadResolvableReport::Action _action;
+    };
+
+
     ///////////////////////////////////////////////////////////////////
     //	class PackageProviderPolicy
     ///////////////////////////////////////////////////////////////////
@@ -52,47 +75,58 @@ namespace zypp
       return false;
     }
 
-
     ///////////////////////////////////////////////////////////////////
     /// \class PackageProvider::Impl
-    /// \brief PackageProvider implementation.
+    /// \brief PackageProvider implementation interface.
     ///////////////////////////////////////////////////////////////////
-    class PackageProvider::Impl : private base::NonCopyable
+    struct PackageProvider::Impl : private base::NonCopyable
     {
+      Impl() {}
+      virtual ~Impl() {}
+
+      /** Provide the package.
+       * The basic workflow.
+       * \throws Exception.
+       */
+      virtual ManagedFile providePackage() const = 0;
+
+      /** Provide the package if it is cached. */
+      virtual ManagedFile providePackageFromCache() const = 0;
+
+      /** Whether the package is cached. */
+      virtual bool isCached() const = 0;
+    };
+
+    ///////////////////////////////////////////////////////////////////
+    /// \class PackageProviderImpl<TPackage>
+    /// \brief PackageProvider implementation for \c Package and \c SrcPackage
+    ///////////////////////////////////////////////////////////////////
+    template <class TPackage>
+    class PackageProviderImpl : public PackageProvider::Impl
+    {
+      typedef typename TPackage::constPtr TPackagePtr;	// Package or SrcPackage
       typedef callback::UserData UserData;
     public:
       /** Ctor taking the Package to provide. */
-      Impl( RepoMediaAccess & access_r,
-	    const Package::constPtr & package_r,
-	    const DeltaCandidates & deltas_r,
-	    const PackageProviderPolicy & policy_r )
+      PackageProviderImpl( RepoMediaAccess & access_r, const TPackagePtr & package_r,
+			   const PackageProviderPolicy & policy_r )
       : _policy( policy_r )
       , _package( package_r )
-      , _deltas( deltas_r )
       , _access( access_r )
       , _retry(false)
       {}
 
-      virtual ~Impl() {}
-
-      /** Factory method providing the appropriate implementation.
-       * Called by PackageProvider ctor. Returned pointer should be
-       * immediately wrapped into a smartpointer.
-       */
-      static Impl * factoryMake( RepoMediaAccess & access_r,
-				 const Package::constPtr & package_r,
-				 const DeltaCandidates & deltas_r,
-				 const PackageProviderPolicy & policy_r );
+      virtual ~PackageProviderImpl() {}
 
     public:
       /** Provide the package.
        * The basic workflow.
        * \throws Exception.
        */
-      ManagedFile providePackage() const;
+      virtual ManagedFile providePackage() const;
 
       /** Provide the package if it is cached. */
-      ManagedFile providePackageFromCache() const
+      virtual ManagedFile providePackageFromCache() const
       {
 	ManagedFile ret( doProvidePackageFromCache() );
 	if ( ! ( ret->empty() ||  _package->repoInfo().keepPackages() ) )
@@ -101,43 +135,47 @@ namespace zypp
       }
 
       /** Whether the package is cached. */
-      bool isCached() const
+      virtual bool isCached() const
       { return ! doProvidePackageFromCache()->empty(); }
 
     protected:
-      typedef PackageProvider::Impl	Base;
+      typedef PackageProviderImpl<TPackage>	Base;
       typedef callback::SendReport<repo::DownloadResolvableReport>	Report;
 
       /** Lookup the final rpm in cache.
        *
-       * A non empty ManagedFile will be returned to the caller.
+       * A cache hit will return a non empty ManagedFile and an empty one on cache miss.
        *
        * \note File disposal depending on the repos keepPackages setting
        * are not set here, but in \ref providePackage or \ref providePackageFromCache.
-       *
-       * \note The provoided default implementation returns an empty ManagedFile
-       * (cache miss).
        */
-      virtual ManagedFile doProvidePackageFromCache() const = 0;
+      ManagedFile doProvidePackageFromCache() const
+      { return ManagedFile( _package->cachedLocation() ); }
 
       /** Actually provide the final rpm.
        * Report start/problem/finish and retry loop are hadled by \ref providePackage.
        * Here you trigger just progress and delta/plugin callbacks as needed.
        *
-       * Proxy methods for progressPackageDownload and failOnChecksum are provided here.
-       * Create similar proxies for other progress callbacks in derived classes and link
-       * it to ProvideFilePolicy for download:
+       * Proxy method for progressPackageDownload is provided here.
        * \code
        * ProvideFilePolicy policy;
        * policy.progressCB( bind( &Base::progressPackageDownload, this, _1 ) );
-       * policy.failOnChecksumErrorCB( bind( &Base::failOnChecksumError, this ) );
        * return _access.provideFile( _package->repoInfo(), loc, policy );
        * \endcode
        *
-       * \note The provoided default implementation retrieves the packages default
+       * \note The provided default implementation retrieves the packages default
        * location.
        */
-      virtual ManagedFile doProvidePackage() const = 0;
+      virtual ManagedFile doProvidePackage() const
+      {
+	ManagedFile ret;
+	OnMediaLocation loc = _package->location();
+
+	ProvideFilePolicy policy;
+	policy.progressCB( bind( &Base::progressPackageDownload, this, _1 ) );
+	policy.fileChecker( bind( &Base::rpmSigFileChecker, this, _1 ) );
+	return _access.provideFile( _package->repoInfo(), loc, policy );
+      }
 
     protected:
       /** Access to the DownloadResolvableReport */
@@ -148,32 +186,140 @@ namespace zypp
       bool progressPackageDownload( int value ) const
       {	return report()->progress( value, _package ); }
 
-      /** Redirect ProvideFilePolicy failOnChecksumError to this if needed. */
-      bool failOnChecksumError() const
-      {
-	std::string package_str = _package->name() + "-" + _package->edition().asString();
 
-	// TranslatorExplanation %s = package being checked for integrity
-	switch ( report()->problem( _package, repo::DownloadResolvableReport::INVALID, str::form(_("Package %s seems to be corrupted during transfer. Do you want to retry retrieval?"), package_str.c_str() ) ) )
+      /** \name Validate a rpm packages signature.
+       *
+       * This is the \ref FileChecker passed down to the \ref Fetcher to validate
+       * a provided rpm package. This builtin checker includes the workflow
+       * communicating with the user in case of a problem with the package
+       * signature.
+       *
+       * \throws RpmSigCheckException if the package is not accepted, propagating
+       * the users decision how to proceed (\ref DownloadResolvableReport::Action).
+       *
+       * \note This check is also needed, if the the rpm is built locally by using
+       * delta rpms! \ref \see RpmPackageProvider
+       */
+      //@{
+      void rpmSigFileChecker( const Pathname & file_r ) const
+      {
+	RepoInfo info = _package->repoInfo();
+	if ( info.pkgGpgCheck() )
 	{
-	  case repo::DownloadResolvableReport::RETRY:
-	    _retry = true;
-	    break;
-	  case repo::DownloadResolvableReport::IGNORE:
-	    ZYPP_THROW(SkipRequestException("User requested skip of corrupted file"));
-	    break;
-	  case repo::DownloadResolvableReport::ABORT:
-	    ZYPP_THROW(AbortRequestException("User requested to abort"));
-	    break;
-	  default:
-	    break;
+	  UserData userData( "pkgGpgCheck" );
+	  ResObject::constPtr roptr( _package );	// gcc6 needs it more explcit. Has problem deducing
+	  userData.set( "ResObject", roptr );		// a type for '_package->asKind<ResObject>()'...
+	  /*legacy:*/userData.set( "Package", roptr->asKind<Package>() );
+	  userData.set( "Localpath", file_r );
+
+	  RpmDb::CheckPackageResult res = RpmDb::CHK_NOKEY;
+	  while ( res == RpmDb::CHK_NOKEY ) {
+	    res = packageSigCheck( file_r, info.pkgGpgCheckIsMandatory(), userData );
+
+            // publish the checkresult, even if it is OK. Apps may want to report something...
+            report()->pkgGpgCheck( userData );
+
+            if ( res == RpmDb::CHK_NOKEY ) {
+              // if the check fails because we don't know the key
+              // we try to resolv it with gpgkey urls from the
+              // repository, if available
+
+              target::rpm::RpmHeader::constPtr hr = target::rpm::RpmHeader::readPackage( file_r );
+              if ( !hr ) {
+                // we did not find any information about the key in the header
+                // this should never happen
+                WAR << "Unable to read package header from " << hr << endl;
+                break;
+              }
+
+              std::string keyID = hr->signatureKeyID();
+              if ( keyID.length() > 0 ) {
+                const ZConfig &conf = ZConfig::instance();
+                Pathname cacheDir = conf.repoManagerRoot() / conf.pubkeyCachePath();
+
+                Pathname myKey = info.provideKey ( keyID, cacheDir );
+                if ( myKey.empty()  )
+                  // if we did not find any keys, there is no point in checking again, break
+                  break;
+
+                callback::SendReport<KeyRingReport> report;
+
+                PublicKey key;
+                try {
+                  key = PublicKey( myKey );
+                } catch ( const Exception &e ) {
+                  ZYPP_CAUGHT(e);
+                  break;
+                }
+
+                if ( !key.isValid() ) {
+                  ERR << "Key [" << keyID << "] from cache: " << cacheDir << " is not valid" << endl;
+                  break;
+                }
+
+                MIL << "Key [" << keyID << "] " << key.name() << " loaded from cache" << endl;
+
+                KeyContext context;
+                context.setRepoInfo( info );
+                if ( ! report->askUserToAcceptPackageKey( key, context ) ) {
+                  break;
+                }
+
+                MIL << "User wants to import key [" << keyID << "] " << key.name() << " from cache" << endl;
+                KeyRing_Ptr theKeyRing = getZYpp()->keyRing();
+                try {
+                  theKeyRing->importKey( key, true );
+                } catch ( const KeyRingException &e ) {
+                  ZYPP_CAUGHT(e);
+                  ERR << "Failed to import key: "<<keyID;
+                  break;
+                }
+              } else {
+                // we did not find any information about the key in the header
+                // this should never happen
+                WAR << "packageSigCheck returned without setting providing missing key information" << endl;
+                break;
+              }
+            }
+          }
+
+	  if ( res != RpmDb::CHK_OK )
+	  {
+	    if ( userData.hasvalue( "Action" ) )	// pkgGpgCheck report provided an user error action
+	    {
+	      resolveSignatureErrorAction( userData.get( "Action", repo::DownloadResolvableReport::ABORT ) );
+	    }
+	    else if ( userData.haskey( "Action" ) )	// pkgGpgCheck requests the default problem report (wo. details)
+	    {
+	      defaultReportSignatureError( res );
+	    }
+	    else					// no advice from user => usedefaults
+	    {
+	      switch ( res )
+	      {
+		case RpmDb::CHK_OK:		// Signature is OK
+		  break;
+
+		case RpmDb::CHK_NOKEY:		// Public key is unavailable
+		case RpmDb::CHK_NOTFOUND:	// Signature is unknown type
+		case RpmDb::CHK_FAIL:		// Signature does not verify
+		case RpmDb::CHK_NOTTRUSTED:	// Signature is OK, but key is not trusted
+		case RpmDb::CHK_ERROR:		// File does not exist or can't be opened
+		case RpmDb::CHK_NOSIG:		// File is unsigned
+		default:
+		  // report problem (w. details), throw if to abort, else retry/ignore
+		  defaultReportSignatureError( res, str::Str() << userData.get<RpmDb::CheckPackageDetail>( "CheckPackageDetail" ) );
+		  break;
+	      }
+	    }
+	  }
 	}
-	return true; // anyway a failure
       }
 
       typedef target::rpm::RpmDb RpmDb;
 
-      RpmDb::CheckPackageResult packageSigCheck( const Pathname & path_r, UserData & userData ) const
+      /** Actual rpm package signature check. */
+      RpmDb::CheckPackageResult packageSigCheck( const Pathname & path_r, bool isMandatory_r, UserData & userData ) const
       {
 	if ( !_target )
 	  _target = getZYpp()->getTarget();
@@ -181,7 +327,14 @@ namespace zypp
 	RpmDb::CheckPackageResult ret = RpmDb::CHK_ERROR;
 	RpmDb::CheckPackageDetail detail;
 	if ( _target )
-	  ret = _target->rpmDb().checkPackage( path_r, detail );
+	{
+	  ret = _target->rpmDb().checkPackageSignature( path_r, detail );
+	  if ( ret == RpmDb::CHK_NOSIG && !isMandatory_r )
+	  {
+	    WAR << "Relax CHK_NOSIG: Config says unsigned packages are OK" << endl;
+	    ret = RpmDb::CHK_OK;
+	  }
+	}
 	else
 	  detail.push_back( RpmDb::CheckPackageDetail::value_type( ret, "OOps. Target is not initialized!" ) );
 
@@ -190,36 +343,38 @@ namespace zypp
 	return ret;
       }
 
-      /** React on signature verrification error user action */
+      /** React on signature verification error user action.
+       * \note: IGNORE == accept insecure file (no SkipRequestException!)
+       */
       void resolveSignatureErrorAction( repo::DownloadResolvableReport::Action action_r ) const
       {
-	// TranslatorExplanation %s = package being checked for integrity
 	switch ( action_r )
 	{
-	  case repo::DownloadResolvableReport::RETRY:
-	    _retry = true;
-	    break;
 	  case repo::DownloadResolvableReport::IGNORE:
-	    WAR << _package->asUserString() << ": " << "User requested skip of insecure file" << endl;
+	    WAR << _package->asUserString() << ": " << "User requested to accept insecure file" << endl;
 	    break;
 	  default:
+	  case repo::DownloadResolvableReport::RETRY:
 	  case repo::DownloadResolvableReport::ABORT:
-	    ZYPP_THROW(AbortRequestException("User requested to abort"));
+	    ZYPP_THROW(RpmSigCheckException(action_r,"Signature verification failed"));
 	    break;
 	}
       }
 
-      /** Default signature verrification error handling. */
-      void defaultReportSignatureError( RpmDb::CheckPackageResult ret ) const
+      /** Default signature verification error handling. */
+      void defaultReportSignatureError( RpmDb::CheckPackageResult ret, const std::string & detail_r = std::string() ) const
       {
-	std::string msg( str::Str() << _package->asUserString() << ": " << _("Signature verification failed") << " " << ret );
-	resolveSignatureErrorAction( report()->problem( _package, repo::DownloadResolvableReport::INVALID, msg ) );
+	str::Str msg;
+	msg << _package->asUserString() << ": " << _("Signature verification failed") << " " << ret;
+	if ( ! detail_r.empty() )
+	  msg << "\n" << detail_r;
+	resolveSignatureErrorAction( report()->problem( _package, repo::DownloadResolvableReport::INVALID, msg.str() ) );
       }
+      //@}
 
     protected:
       PackageProviderPolicy	_policy;
-      Package::constPtr		_package;
-      DeltaCandidates		_deltas;
+      TPackagePtr		_package;
       RepoMediaAccess &		_access;
 
     private:
@@ -242,25 +397,8 @@ namespace zypp
     };
     ///////////////////////////////////////////////////////////////////
 
-    /** Default implementation (cache miss). */
-    ManagedFile PackageProvider::Impl::doProvidePackageFromCache() const
-    { return ManagedFile(); }
-
-    /** Default implementation (provide full package) */
-    ManagedFile PackageProvider::Impl::doProvidePackage() const
-    {
-      ManagedFile ret;
-      OnMediaLocation loc = _package->location();
-
-      ProvideFilePolicy policy;
-      policy.progressCB( bind( &Base::progressPackageDownload, this, _1 ) );
-      policy.failOnChecksumErrorCB( bind( &Base::failOnChecksumError, this ) );
-      return _access.provideFile( _package->repoInfo(), loc, policy );
-    }
-
-    ///////////////////////////////////////////////////////////////////
-
-    ManagedFile PackageProvider::Impl::providePackage() const
+    template <class TPackage>
+    ManagedFile PackageProviderImpl<TPackage>::providePackage() const
     {
       ScopedGuard guardReport( newReport() );
 
@@ -284,11 +422,11 @@ namespace zypp
 	  const OnMediaLocation & loc( _package->location() );
 	  if ( ! loc.checksum().empty() )	// no cache hit without checksum
 	  {
-	    PathInfo pi( topCache.repoPackagesCachePath / info.packagesPath().basename() / loc.filename() );
+	    PathInfo pi( topCache.repoPackagesCachePath / info.packagesPath().basename() / info.path() / loc.filename() );
 	    if ( pi.isExist() && loc.checksum() == CheckSum( loc.checksum().type(), std::ifstream( pi.c_str() ) ) )
 	    {
 	      report()->start( _package, pi.path().asFileUrl() );
-	      const Pathname & dest( info.packagesPath() / loc.filename() );
+	      const Pathname & dest( info.packagesPath() / info.path() / loc.filename() );
 	      if ( filesystem::assert_dir( dest.dirname() ) == 0 && filesystem::hardlinkCopy( pi.path(), dest ) == 0 )
 	      {
 		ret = ManagedFile( dest );
@@ -310,6 +448,7 @@ namespace zypp
 
       MIL << "provide Package " << _package << endl;
       Url url = * info.baseUrlsBegin();
+      try {
       do {
         _retry = false;
 	if ( ! ret->empty() )
@@ -318,66 +457,67 @@ namespace zypp
 	  ret.reset();
 	}
         report()->start( _package, url );
-        try  // ELIMINATE try/catch by providing a log-guard
+        try
           {
             ret = doProvidePackage();
-
-	    if ( info.pkgGpgCheck() )
-	    {
-	      UserData userData( "pkgGpgCheck" );
-	      userData.set( "Package", _package );
-	      userData.set( "Localpath", ret.value() );
-	      RpmDb::CheckPackageResult res = packageSigCheck( ret, userData );
-	      // publish the checkresult, even if it is OK. Apps may want to report something...
-	      report()->pkgGpgCheck( userData );
-
-	      if ( res != RpmDb::CHK_OK )
-	      {
-		if ( userData.hasvalue( "Action" ) )	// pkgGpgCheck report provided an user error action
-		{
-		  resolveSignatureErrorAction( userData.get( "Action", repo::DownloadResolvableReport::ABORT ) );
-		}
-		else if ( userData.haskey( "Action" ) )	// pkgGpgCheck requests the default problem report
-		{
-		  defaultReportSignatureError( res );
-		}
-		else					// no advice from user => usedefaults
-		{
-		  switch ( res )
-		  {
-		    case RpmDb::CHK_OK:		// Signature is OK
-		      break;
-
-		    case RpmDb::CHK_NOKEY:	// Public key is unavailable
-		    case RpmDb::CHK_NOTFOUND:	// Signature is unknown type
-		    case RpmDb::CHK_FAIL:	// Signature does not verify
-		    case RpmDb::CHK_NOTTRUSTED:	// Signature is OK, but key is not trusted
-		    case RpmDb::CHK_ERROR:	// File does not exist or can't be opened
-		    default:
-		      // report problem, throw if to abort, else retry/ignore
-		      defaultReportSignatureError( res );
-		      break;
-		  }
-		}
-	      }
-	    }
           }
         catch ( const UserRequestException & excpt )
           {
-            // UserRequestException e.g. from failOnChecksumError was already reported.
             ERR << "Failed to provide Package " << _package << endl;
-            if ( ! _retry )
-              {
-                ZYPP_RETHROW( excpt );
-              }
+	    if ( ! _retry )
+	      ZYPP_RETHROW( excpt );
           }
+	catch ( const RpmSigCheckException & excpt )
+	  {
+	    ERR << "Failed to provide Package " << _package << endl;
+	    if ( ! _retry )
+	    {
+	      // Signature verification error was already reported by the
+	      // rpmSigFileChecker. Just handle the users action decision:
+	      switch ( excpt.action() )
+	      {
+		case repo::DownloadResolvableReport::RETRY:
+		  _retry = true;
+		  break;
+		case repo::DownloadResolvableReport::IGNORE:
+		  ZYPP_THROW(SkipRequestException("User requested skip of corrupted file"));
+		  break;
+		default:
+		case repo::DownloadResolvableReport::ABORT:
+		  ZYPP_THROW(AbortRequestException("User requested to abort"));
+		  break;
+	      }
+	    }
+	  }
+        catch ( const FileCheckException & excpt )
+          {
+	    ERR << "Failed to provide Package " << _package << endl;
+	    if ( ! _retry )
+	    {
+	      const std::string & package_str = _package->asUserString();
+	      // TranslatorExplanation %s = package being checked for integrity
+	      switch ( report()->problem( _package, repo::DownloadResolvableReport::INVALID, str::form(_("Package %s seems to be corrupted during transfer. Do you want to retry retrieval?"), package_str.c_str() ) ) )
+	      {
+		case repo::DownloadResolvableReport::RETRY:
+		  _retry = true;
+		  break;
+		case repo::DownloadResolvableReport::IGNORE:
+		  ZYPP_THROW(SkipRequestException("User requested skip of corrupted file"));
+		  break;
+		default:
+		case repo::DownloadResolvableReport::ABORT:
+		  ZYPP_THROW(AbortRequestException("User requested to abort"));
+		  break;
+	      }
+	    }
+	  }
         catch ( const Exception & excpt )
           {
             ERR << "Failed to provide Package " << _package << endl;
             if ( ! _retry )
-              {
+	    {
                 // Aything else gets reported
-                std::string package_str = _package->name() + "-" + _package->edition().asString();
+                const std::string & package_str = _package->asUserString();
 
                 // TranslatorExplanation %s = name of the package being processed.
                 std::string detail_str( str::form(_("Failed to provide Package %s. Do you want to retry retrieval?"), package_str.c_str() ) );
@@ -389,18 +529,22 @@ namespace zypp
                         _retry = true;
                         break;
                       case repo::DownloadResolvableReport::IGNORE:
-                        ZYPP_THROW(SkipRequestException("User requested skip of corrupted file", excpt));
-                        break;
-                      case repo::DownloadResolvableReport::ABORT:
-                        ZYPP_THROW(AbortRequestException("User requested to abort", excpt));
+                        ZYPP_THROW(SkipRequestException("User requested skip of file", excpt));
                         break;
                       default:
-                        ZYPP_RETHROW( excpt );
+                      case repo::DownloadResolvableReport::ABORT:
+                        ZYPP_THROW(AbortRequestException("User requested to abort", excpt));
                         break;
                 }
               }
           }
       } while ( _retry );
+      } catch(...){
+	// bsc#1045735: Be sure no invalid files stay in the cache!
+	if ( ! ret->empty() )
+	  ret.setDispose( filesystem::unlink );
+	throw;
+      }
 
       report()->finish( _package, repo::DownloadResolvableReport::NO_ERROR, std::string() );
       MIL << "provided Package " << _package << " at " << ret << endl;
@@ -410,21 +554,20 @@ namespace zypp
 
     ///////////////////////////////////////////////////////////////////
     /// \class RpmPackageProvider
-    /// \brief RPM PackageProvider implementation.
+    /// \brief RPM PackageProvider implementation (with deltarpm processing).
     ///////////////////////////////////////////////////////////////////
-    class RpmPackageProvider : public PackageProvider::Impl
+    class RpmPackageProvider : public PackageProviderImpl<Package>
     {
     public:
       RpmPackageProvider( RepoMediaAccess & access_r,
 			  const Package::constPtr & package_r,
 			  const DeltaCandidates & deltas_r,
 			  const PackageProviderPolicy & policy_r )
-      : PackageProvider::Impl( access_r, package_r, deltas_r, policy_r )
+      : PackageProviderImpl<Package>( access_r, package_r, policy_r )
+      , _deltas( deltas_r )
       {}
 
     protected:
-      virtual ManagedFile doProvidePackageFromCache() const;
-
       virtual ManagedFile doProvidePackage() const;
 
     private:
@@ -440,27 +583,18 @@ namespace zypp
 
       bool queryInstalled( const Edition & ed_r = Edition() ) const
       { return _policy.queryInstalled( _package->name(), ed_r, _package->arch() ); }
+
+    private:
+      DeltaCandidates		_deltas;
     };
     ///////////////////////////////////////////////////////////////////
 
-    ManagedFile RpmPackageProvider::doProvidePackageFromCache() const
-    {
-      return ManagedFile( _package->cachedLocation() );
-    }
-
     ManagedFile RpmPackageProvider::doProvidePackage() const
     {
-      Url url;
-      RepoInfo info = _package->repoInfo();
-      // FIXME we only support the first url for now.
-      if ( info.baseUrlsEmpty() )
-        ZYPP_THROW(Exception("No url in repository."));
-      else
-        url = * info.baseUrlsBegin();
-
       // check whether to process patch/delta rpms
+      // FIXME we only check the first url for now.
       if ( ZConfig::instance().download_use_deltarpm()
-	&& ( url.schemeIsDownloading() || ZConfig::instance().download_use_deltarpm_always() ) )
+	&& ( _package->repoInfo().url().schemeIsDownloading() || ZConfig::instance().download_use_deltarpm_always() ) )
       {
 	std::list<DeltaRpm> deltaRpms;
 	_deltas.deltaRpms( _package ).swap( deltaRpms );
@@ -513,70 +647,83 @@ namespace zypp
           return ManagedFile();
         }
 
-      // build the package and put it into the cache
-      Pathname destination( _package->repoInfo().packagesPath() / _package->location().filename() );
+      // Build the package
+      Pathname cachedest( _package->repoInfo().packagesPath() / _package->repoInfo().path() / _package->location().filename() );
+      Pathname builddest( cachedest.extend( ".drpm" ) );
 
-      if ( ! applydeltarpm::provide( delta, destination,
+      if ( ! applydeltarpm::provide( delta, builddest,
                                      bind( &RpmPackageProvider::progressDeltaApply, this, _1 ) ) )
         {
           report()->problemDeltaApply( _("applydeltarpm failed.") );
           return ManagedFile();
         }
+      ManagedFile builddestCleanup( builddest, filesystem::unlink );
       report()->finishDeltaApply();
 
-      return ManagedFile( destination, filesystem::unlink );
+      // Check and move it into the cache
+      // Here the rpm itself is ready. If the packages sigcheck fails, it
+      // makes no sense to return a ManagedFile() and fallback to download the
+      // full rpm. It won't be different. So let the exceptions escape...
+      rpmSigFileChecker( builddest );
+      if ( filesystem::hardlinkCopy( builddest, cachedest ) != 0 )
+	ZYPP_THROW( Exception( str::Str() << "Can't hardlink/copy " << builddest << " to " << cachedest ) );
+
+      return ManagedFile( cachedest, filesystem::unlink );
     }
 
-#if 0
-    ///////////////////////////////////////////////////////////////////
-    /// \class PluginPackageProvider
-    /// \brief Plugin PackageProvider implementation.
-    ///
-    /// Basically downloads the default package and calls a
-    /// 'stem'2rpm plugin to cteate the final .rpm package.
-    ///////////////////////////////////////////////////////////////////
-    class PluginPackageProvider : public PackageProvider::Impl
-    {
-    public:
-      PluginPackageProvider( const std::string & stem_r,
-			     RepoMediaAccess & access_r,
-			     const Package::constPtr & package_r,
-			     const DeltaCandidates & deltas_r,
-			     const PackageProviderPolicy & policy_r )
-      : Base( access_r, package_r, deltas_r, policy_r )
-      {}
-
-    protected:
-      virtual ManagedFile doProvidePackageFromCache() const
-      {
-	return Base::doProvidePackageFromCache();
-      }
-
-      virtual ManagedFile doProvidePackage() const
-      {
-	return Base::doProvidePackage();
-      }
-    };
-    ///////////////////////////////////////////////////////////////////
-#endif
-
     ///////////////////////////////////////////////////////////////////
     //	class PackageProvider
     ///////////////////////////////////////////////////////////////////
-
-    PackageProvider::Impl * PackageProvider::Impl::factoryMake( RepoMediaAccess & access_r,
-								const Package::constPtr & package_r,
-								const DeltaCandidates & deltas_r,
-								const PackageProviderPolicy & policy_r )
+    namespace factory
     {
-      return new RpmPackageProvider( access_r, package_r, deltas_r, policy_r );
-    }
+      inline PackageProvider::Impl * make( RepoMediaAccess & access_r, const PoolItem & pi_r,
+					   const DeltaCandidates & deltas_r,
+					   const PackageProviderPolicy & policy_r )
+      {
+	if ( pi_r.isKind<Package>() )
+	  return new RpmPackageProvider( access_r, pi_r->asKind<Package>(), deltas_r, policy_r );
+	else if ( pi_r.isKind<SrcPackage>() )
+	  return new PackageProviderImpl<SrcPackage>( access_r, pi_r->asKind<SrcPackage>(), policy_r );
+	else
+	  ZYPP_THROW( Exception( str::Str() << "Don't know how to cache non-package " << pi_r.asUserString() ) );
+      }
 
+      inline PackageProvider::Impl * make( RepoMediaAccess & access_r, const PoolItem & pi_r,
+						  const PackageProviderPolicy & policy_r )
+      {
+	if ( pi_r.isKind<Package>() )
+	  return new PackageProviderImpl<Package>( access_r, pi_r->asKind<Package>(), policy_r );
+	else if ( pi_r.isKind<SrcPackage>() )
+	  return new PackageProviderImpl<SrcPackage>( access_r, pi_r->asKind<SrcPackage>(), policy_r );
+	else
+	  ZYPP_THROW( Exception( str::Str() << "Don't know how to cache non-package " << pi_r.asUserString() ) );
+      }
+
+      inline PackageProvider::Impl * make( RepoMediaAccess & access_r, const Package::constPtr & package_r,
+					   const DeltaCandidates & deltas_r,
+					   const PackageProviderPolicy & policy_r )
+      { return new RpmPackageProvider( access_r, package_r, deltas_r, policy_r ); }
+
+    } // namespace factory
+    ///////////////////////////////////////////////////////////////////
+
+    PackageProvider::PackageProvider( RepoMediaAccess & access_r, const PoolItem & pi_r,
+				      const DeltaCandidates & deltas_r, const PackageProviderPolicy & policy_r )
+
+    : _pimpl( factory::make( access_r, pi_r, deltas_r, policy_r ) )
+    {}
+
+    PackageProvider::PackageProvider( RepoMediaAccess & access_r, const PoolItem & pi_r,
+				      const PackageProviderPolicy & policy_r )
+    : _pimpl( factory::make( access_r, pi_r, policy_r ) )
+    {}
+
+    /* legacy */
     PackageProvider::PackageProvider( RepoMediaAccess & access_r,
 				      const Package::constPtr & package_r,
 				      const DeltaCandidates & deltas_r,
 				      const PackageProviderPolicy & policy_r )
-    : _pimpl( Impl::factoryMake( access_r, package_r, deltas_r, policy_r ) )
+    : _pimpl( factory::make( access_r, package_r, deltas_r, policy_r ) )
     {}
 
     PackageProvider::~PackageProvider()
