@@ -114,7 +114,7 @@ namespace zypp
       IdString ident() const;
 
       /** The ResObjects kind. */
-      ResObject::Kind kind() const;
+      ResKind kind() const;
 
       /** The ResObjects name.  */
       const std::string & name() const;
@@ -150,7 +150,10 @@ namespace zypp
        */
       PoolItem highestAvailableVersionObj() const;
 
-     /** \c True if \a rhs is installed and one with the same content is available.
+      /** Whether this \ref ident is flagged as AutoInstalled. */
+      bool identIsAutoInstalled() const;
+
+      /** \c True if \a rhs is installed and one with the same content is available.
        * Basically the same name, edition, arch, vendor and buildtime.
        * \see \ref sat::Solvable::identical
        */
@@ -192,9 +195,9 @@ namespace zypp
        *   Package::constPtr p( mySelectable.installedAsKind<Package>() );
        * \endcode
       */
-      template<class _Res>
-      typename ResTraits<_Res>::constPtrType installedAsKind() const
-      { return asKind<_Res>( candidateObj() ); }
+      template<class TRes>
+      typename ResTraits<TRes>::constPtrType installedAsKind() const
+      { return asKind<TRes>( candidateObj() ); }
 
       /** Return the \ref candidateObj resolvable casted to a specific kind.
        * \code
@@ -202,9 +205,9 @@ namespace zypp
        *   Package::constPtr p( mySelectable.candidateAsKind<Package>() );
        * \endcode
       */
-      template<class _Res>
-      typename ResTraits<_Res>::constPtrType candidateAsKind() const
-      { return asKind<_Res>( candidateObj() ); }
+      template<class TRes>
+      typename ResTraits<TRes>::constPtrType candidateAsKind() const
+      { return asKind<TRes>( candidateObj() ); }
 
       /** Set a candidate (out of available objects).
        * \return The new candidate, or NULL if choice was invalid
@@ -243,6 +246,8 @@ namespace zypp
       available_size_type availableSize() const;
       available_iterator availableBegin() const;
       available_iterator availableEnd() const;
+      inline Iterable<available_iterator>  available() const
+      { return makeIterable( availableBegin(), availableEnd() ); }
       //@}
 
       ////////////////////////////////////////////////////////////////////////
@@ -255,7 +260,9 @@ namespace zypp
       installed_size_type installedSize() const;
       installed_iterator installedBegin() const;
       installed_iterator installedEnd() const;
-      //}
+      inline Iterable<installed_iterator> installed() const
+      { return makeIterable( installedBegin(), installedEnd() ); }
+      //@}
 
       ////////////////////////////////////////////////////////////////////////
 
@@ -268,7 +275,21 @@ namespace zypp
       picklist_size_type picklistSize() const;
       picklist_iterator picklistBegin() const;
       picklist_iterator picklistEnd() const;
-      //}
+      inline Iterable<picklist_iterator> picklist() const
+      { return makeIterable( picklistBegin(), picklistEnd() ); }
+
+      /** Returned by \ref picklistPos if the Item does not belong to the picklist. */
+      static constexpr const picklist_size_type picklistNoPos = picklist_size_type(-1);
+
+      /** Return the position of \a pi_r in the piclist or \ref picklistNoPos.
+       * \note \ref picklistNoPos is returned if you pass an installed Poolitem,
+       * whci has an \ref identicalAvailableObj.
+       */
+      picklist_size_type picklistPos( const PoolItem & pi_r ) const;
+
+      /** \overload taking a sat::Solvable */
+      picklist_size_type picklistPos( const sat::Solvable & solv_r ) const;
+      //@}
 
       ////////////////////////////////////////////////////////////////////////
 
@@ -314,8 +335,9 @@ namespace zypp
        * install will deselect any other.
        */
       //@{
-      /** Whether different versions of this package can be installed at the same time.
-       * Per default \c false. \see also \ref ZConfig::multiversion.
+      /** Whether at least one of the available packages has multiversionInstall set.
+       * \see \ref Solvable::multiversionInstall
+       * \see also \ref ZConfig::multiversion.
        */
       bool multiversionInstall() const;
 
